from typing import Any, List, Union

from django.db import models
from queryset_sequence import QuerySetSequence

from crossref_bpp.utils import json_format_with_wrap, perform_trigram_search
from import_common.core import (
    matchuj_autora,
    matchuj_wydawce,
    normalize_zrodlo_nazwa_for_db_lookup,
    normalize_zrodlo_skrot_for_db_lookup,
    normalized_db_title,
    normalized_db_zrodlo_nazwa,
    normalized_db_zrodlo_skrot,
)
from import_common.normalization import (
    normalize_doi,
    normalize_first_name,
    normalize_issn,
    normalize_last_name,
    normalize_orcid,
    normalize_publisher,
)

from django.utils.datastructures import CaseInsensitiveMapping
from django.utils.functional import cached_property

from bpp.models import (
    Autor,
    Charakter_Formalny,
    Jezyk,
    Licencja_OpenAccess,
    Rekord,
    Wydawca,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
    Zrodlo,
)


class StatusPorownania(models.TextChoices):
    DOKLADNE = "ok", "dokładne (identyczne dane po obu stronach)"
    LUZNE = "luzne", "luźne (może być nieprawidłowe)"
    WYMAGA_INGERENCJI = (
        "user",
        "wymaga ręcznego wybrania (dwa lub więcej dokładne wyniki)",
    )
    BRAK = "brak", "brak porównania"
    BLAD = "blad", "błąd porównania - pusty lub niepoprawny parametr wejściowy"


class WynikPorownania:
    def __init__(
        self,
        status: StatusPorownania,
        opis: str = "",
        rekordy: [
            Union[
                List[
                    models.Model,
                ],
                None,
            ]
        ] = None,
    ):
        self.status = status
        self.opis = opis
        self.rekordy = rekordy

    @cached_property
    def rekord_po_stronie_bpp(self):
        """Zwróć pierwszy rekord z self.rekordy jeżeli wynik porównania jest dokładny lub luźny"""
        ile_rekordow = 0
        if hasattr(
            self.rekordy,
            "exists",
        ):
            ile_rekordow = self.rekordy.count()
            pierwszy_rekord = self.rekordy.first()
        elif hasattr(self.rekordy, "__len__"):
            ile_rekordow = len(self.rekordy)
            pierwszy_rekord = self.rekordy[0]

        if (
            self.status in (StatusPorownania.DOKLADNE, StatusPorownania.LUZNE)
            and ile_rekordow == 1
        ):
            return pierwszy_rekord


NIEPRAWIDLOWY_FORMAT_DANYCH = WynikPorownania(
    StatusPorownania.BLAD, "nieprawidłowy format danych"
)
BRAK_DOPASOWANIA = WynikPorownania(StatusPorownania.BLAD, "brak dopasowania")


class Komparator:
    class atrybuty:
        do_porownania_rekordu = [
            "DOI",
            "alternative-id",
            "title",
        ]

        do_zmatchowania = do_porownania_rekordu + [
            "ISSN",
            "issn-type",
            "link",
            "short-container-title",
            "container-title",
            "type",
            "language",
            "license",
            "publisher",
            "URL",
            "resource",
            "author",
        ]

        do_skopiowania = [
            "abstract",
            "issue",
            "page",
            "subject",
            "volume",
        ]

        ignorowane = [
            "content-domain",
            "created",
            "deposited",
            "indexed",
            "is-referenced-by-count",
            "issued",
            "journal-issue",
            "member",
            "original-title",
            "published",
            "published-online",
            "reference",
            "reference-count",
            "references-count",
            "relation",
            "short-title",
            "source",
            "subtitle",
            "prefix",
            "score",
        ]

        wszystkie = do_skopiowania + do_zmatchowania + ignorowane

    @classmethod
    def porownaj(cls, atrybut: str, wartosc_z_crossref: Any):
        atrybut = atrybut.replace("-", "_")
        fn = getattr(cls, f"porownaj_{atrybut}", None)
        if fn is None:
            return WynikPorownania(
                StatusPorownania.BLAD,
                "brak funkcji w oprogramowaniu BPP do porównania tego atrybutu",
            )
        return fn(wartosc_z_crossref)

    @classmethod
    def znajdz(cls, *args, **kwargs):
        return Rekord.objects.filter(*args, **kwargs).order_by("-ostatnio_zmieniony")[
            :10
        ]

    @classmethod
    def znajdz_w_tabelach(cls, *args, **kwargs):
        qs1 = Wydawnictwo_Zwarte.objects.filter(*args, **kwargs).order_by(
            "-ostatnio_zmieniony"
        )
        qs2 = Wydawnictwo_Ciagle.objects.filter(*args, **kwargs).order_by(
            "-ostatnio_zmieniony"
        )
        return QuerySetSequence(qs1, qs2).order_by("-ostatnio_zmieniony")[:10]

    @classmethod
    def porownaj_DOI(cls, wartosc):
        doi = normalize_doi(wartosc)

        if not doi:
            return WynikPorownania(StatusPorownania.BLAD, "puste DOI")

        ile = cls.znajdz(doi__iexact=doi)

        if not ile.exists():
            return WynikPorownania(StatusPorownania.BRAK, "brak takiego DOI w bazie")

        if ile.count() == 1:
            return WynikPorownania(
                StatusPorownania.DOKLADNE,
                "w BPP jest dokładnie jeden taki rekord o takim DOI",
                rekordy=ile,
            )

        # wiecej, jak jeden rekord
        return WynikPorownania(
            StatusPorownania.WYMAGA_INGERENCJI,
            "więcej, nż jeden rekord w BPP o takim DOI",
            rekordy=ile,
        )

    porownaj_alternative_id = porownaj_DOI

    @classmethod
    def porownaj_ISSN(cls, wartosc):
        issn = normalize_issn(wartosc)
        if issn is None:
            return WynikPorownania(StatusPorownania.BLAD, "niepirawidłowy ISSN")

        for fld in "issn", "e_issn":
            kw = {f"{fld}__icontains": issn}

            ile = cls.znajdz_w_tabelach(**kw)

            if not ile.exists():
                continue

            if ile.count() == 1:
                return WynikPorownania(
                    StatusPorownania.DOKLADNE,
                    "w BPP jest dokładnie jeden taki rekord o takim ISSN lub E-ISSN",
                    rekordy=ile,
                )

            # wiecej, jak jeden rekord
            return WynikPorownania(
                StatusPorownania.WYMAGA_INGERENCJI,
                "więcej, nż jeden rekord w BPP o takim ISSN lub E-ISSN",
                rekordy=ile,
            )

        return WynikPorownania(
            StatusPorownania.BRAK, "brak rekordów o takim ISSN lub E-ISSN w BPP"
        )

    @classmethod
    def porownaj_issn_type(cls, wartosc):
        return cls.porownaj_ISSN(wartosc.get("value"))

    @classmethod
    def porownaj_URL(cls, wartosc):
        if (
            wartosc is None
            or not isinstance(wartosc, str)
            or not wartosc
            or not wartosc.strip()
        ):
            return WynikPorownania(StatusPorownania.BLAD, "nieprawidłowy URL")

        if (
            wartosc.lower().find("dx.doi.org/") > -1
            or wartosc.lower().find("doi.org/") > -1
        ):
            return WynikPorownania(
                StatusPorownania.BLAD,
                "URL wyglada na zakodowany numer DOI, nie wykonuję porównania",
            )

        for fld in "www", "public_www":
            kw = {f"{fld}__icontains": wartosc}

            ile = cls.znajdz_w_tabelach(**kw)

            if not ile.exists():
                continue

            if ile.count() == 1:
                return WynikPorownania(
                    StatusPorownania.DOKLADNE,
                    "w BPP jest dokładnie jeden taki rekord o takim adresie WWW",
                    rekordy=ile,
                )

            # wiecej, jak jeden rekord
            return WynikPorownania(
                StatusPorownania.WYMAGA_INGERENCJI,
                "więcej, nż jeden rekord w BPP o takim adresie WWW",
                rekordy=ile,
            )

        return WynikPorownania(
            StatusPorownania.BRAK, "brak rekordów o takiej stronie WWW w BPP"
        )

    @classmethod
    def porownaj_link(cls, wartosc):
        return cls.porownaj_URL(wartosc.get("URL"))

    @classmethod
    def porownaj_resource(cls, wartosc):
        return cls.porownaj_link(wartosc.get("primary"))

    @classmethod
    def porownaj_author(cls, wartosc):
        if wartosc is None or not isinstance(wartosc, dict):
            return NIEPRAWIDLOWY_FORMAT_DANYCH

        wartosc = CaseInsensitiveMapping(wartosc)

        orcid = normalize_orcid(wartosc.get("orcid"))
        if orcid:
            ret = None
            try:
                ret = Autor.objects.get(orcid=orcid)
            except Autor.DoesNotExist:
                pass
            if ret:
                return WynikPorownania(
                    StatusPorownania.DOKLADNE, "dopasowanie po ORCID", rekordy=[ret]
                )

        nazwisko = normalize_last_name(wartosc.get("family"))
        imiona = normalize_first_name(wartosc.get("given"))
        if nazwisko and imiona:
            ret = matchuj_autora(imiona, nazwisko)
            if ret:
                return WynikPorownania(
                    StatusPorownania.LUZNE,
                    "znaleziono dokładnie jednego autora po imieniu i nazwisku",
                    rekordy=[
                        ret,
                    ],
                )

            # Brak jednego autora, ale moze jest wielu? Funkcja matchuj_autora nie zwraca
            # takich danych:

            q = Autor.objects.filter(
                nazwisko__icontains=nazwisko, imiona__icontains=imiona
            )

            if q.exists():
                addendum = ""
                MAX_AUT = 10
                if q.count() >= MAX_AUT:
                    addendum = (
                        f" - w sumie {q.count()} rekordow, pokazuje pierwsze {MAX_AUT}"
                    )

                msg = "kilku autorów - luźne porównanie po imieniu i nazwisku"
                status = StatusPorownania.WYMAGA_INGERENCJI
                if q.count() == 1:
                    msg = "jeden autor - luźne porównanie po imieniu i nazwisku"
                    status = StatusPorownania.LUZNE

                return WynikPorownania(
                    status,
                    f"{msg}{addendum}",
                    rekordy=q[:MAX_AUT],
                )

        return BRAK_DOPASOWANIA

    @classmethod
    def porownaj_short_container_title(cls, wartosc):
        poszukiwania = [wartosc]
        if wartosc.lower().startswith("the "):
            poszukiwania.append(wartosc[4:])

        for ciag in poszukiwania:

            tgrm = perform_trigram_search(
                Zrodlo.objects.exclude(skrot="").exclude(skrot=None),
                normalized_db_zrodlo_skrot,
                normalize_zrodlo_skrot_for_db_lookup(ciag),
            )
            if tgrm:
                return WynikPorownania(
                    StatusPorownania.LUZNE,
                    "luźne porównanie tytułu wg funkcji podobieństwa trygramów",
                    rekordy=tgrm,
                )

        return BRAK_DOPASOWANIA

    @classmethod
    def porownaj_container_title(cls, wartosc):
        poszukiwania = [wartosc]
        if wartosc.lower().startswith("the "):
            poszukiwania.append(wartosc[4:])

        for ciag in poszukiwania:
            tgrm = perform_trigram_search(
                Zrodlo.objects.exclude(nazwa="").exclude(nazwa=None),
                normalized_db_zrodlo_nazwa,
                normalize_zrodlo_nazwa_for_db_lookup(ciag),
            )
            if tgrm:
                return WynikPorownania(
                    StatusPorownania.LUZNE,
                    "luźne porównanie tytułu wg funkcji podobieństwa trygramów",
                    rekordy=tgrm,
                )

        return BRAK_DOPASOWANIA

    @classmethod
    def porownaj_publisher(cls, wartosc):
        wartosc = normalize_publisher(wartosc)
        if wartosc is None:
            return NIEPRAWIDLOWY_FORMAT_DANYCH

        wartosci = [
            wartosc,
        ]
        if wartosc.find("/") > -1:
            a, b = wartosc.split("/", 1)
            wartosci.append(a)
            wartosci.append(b)

        mozliwe_wartosci = set()

        for wartosc in wartosci:
            ret = matchuj_wydawce(wartosc, similarity=0.65)

            if ret is None:

                t = Wydawca.objects.filter(nazwa__istartswith=wartosc).filter()
                if t.count() == 1:
                    mozliwe_wartosci.add(t)
                continue

            return WynikPorownania(
                StatusPorownania.LUZNE,
                "luźne dopasowanie po nazwie",
                rekordy=[
                    ret,
                ],
            )

        if mozliwe_wartosci:
            if len(mozliwe_wartosci) == 1:
                return WynikPorownania(
                    StatusPorownania.LUZNE,
                    "luźne dopasowanie po początku nazwy",
                    rekordy=list(mozliwe_wartosci),
                )

            return WynikPorownania(
                StatusPorownania.WYMAGA_INGERENCJI,
                "ekstremalnie luźne dopasowanie po początku nazwy, pasuje do kilku wydawców",
                rekordy=list(mozliwe_wartosci),
            )

        if ret is None:
            return BRAK_DOPASOWANIA

    @classmethod
    def porownaj_title(cls, wartosc):
        ret = Rekord.objects.filter(tytul_oryginalny__istartswith=wartosc)
        if not ret.exists():

            last_chance = perform_trigram_search(
                Rekord.objects.all(), normalized_db_title, wartosc.lower().strip()
            )
            if last_chance:
                return WynikPorownania(
                    StatusPorownania.LUZNE,
                    "luźne porównanie tytułu wg funkcji podobieństwa trygramów",
                    rekordy=last_chance,
                )

            return BRAK_DOPASOWANIA

        count = ret.count()

        if count == 1:
            return WynikPorownania(
                StatusPorownania.LUZNE,
                "znaleziono 1 pracę z tytułem zaczynającym się podobnie",
                rekordy=list(ret),
            )

        w = "rekordów"
        if count <= 4:
            w = "rekordy"
        return WynikPorownania(
            StatusPorownania.WYMAGA_INGERENCJI,
            f"znaleziono {count} {w} z tytułem zaczynającym się podobnie",
            rekordy=list(ret),
        )

    @classmethod
    def porownaj_type(cls, wartosc):
        if wartosc not in Charakter_Formalny.CHARAKTER_CROSSREF.labels:
            return WynikPorownania(
                StatusPorownania.BLAD,
                f'w systemie BPP nie zdefiniowano wartosci dla typu charakteru formalnego "{wartosc}",'
                f" prosimy o zgłoszenie tej sytuacji autorowi programu. ",
            )

        try:
            c = Charakter_Formalny.objects.get(
                charakter_crossref=getattr(
                    Charakter_Formalny.CHARAKTER_CROSSREF,
                    wartosc.upper().replace("-", "_"),
                )
            )
            return WynikPorownania(
                StatusPorownania.DOKLADNE, "określony charakter formalny", rekordy=[c]
            )
        except Charakter_Formalny.DoesNotExist:
            return WynikPorownania(
                StatusPorownania.BLAD,
                f'w systemie BPP nie zdefiniowano wartosci dla typu charakteru formalnego "{wartosc}",'
                f" wejdz w Dane Systemowe -> Charaktery formalne i wybierz typ charakteru crossref "
                f"dla któregoś z charakteru. ",
            )

    @classmethod
    def porownaj_license(cls, wartosc):
        if wartosc.get("URL"):
            url = wartosc.get("URL", "")
            try:
                skrot = "CC-" + (
                    url.split("creativecommons.org/licenses/")[1].split("/")[0].upper()
                )
            except (IndexError, TypeError, ValueError):
                return WynikPorownania(
                    StatusPorownania.BLAD,
                    f"system BPP nie umie dopasować rodzaju licencji dla {url}, proszę zgłosić"
                    f" ten fakt do autora oprogramowania. ",
                )
            try:
                licencja = Licencja_OpenAccess.objects.get(skrot=skrot)
            except Licencja_OpenAccess.DoesNotExist:
                return WynikPorownania(
                    StatusPorownania.BLAD,
                    f"w systemie BPP nie znaleziono licencji ze skrótem {skrot}, proszę o dopisanie",
                )

            return WynikPorownania(
                StatusPorownania.DOKLADNE,
                "okreslony rodzaj licencji",
                rekordy=[licencja],
            )

    @classmethod
    def porownaj_language(cls, wartosc):
        if wartosc not in Jezyk.SKROT_CROSSREF.values:
            return WynikPorownania(
                StatusPorownania.BLAD,
                f'w systemie BPP nie zdefiniowano wartosci dla typu jezyka "{wartosc}",'
                f" prosimy o zgłoszenie tej sytuacji autorowi programu. ",
            )

        try:
            c = Jezyk.objects.get(skrot_crossref=wartosc)
            return WynikPorownania(
                StatusPorownania.DOKLADNE, "określony jezyk", rekordy=[c]
            )
        except Jezyk.DoesNotExist:
            return WynikPorownania(
                StatusPorownania.BLAD,
                f'w systemie BPP nie zdefiniowano wartosci dla typu jezyka "{wartosc}",'
                f" wejdz w Dane Systemowe -> Języki i wybierz typ charakteru crossref "
                f"dla któregoś z języków. ",
            )

    @classmethod
    def utworz_dane_porownania(cls, json_data):
        dane_porownania = []

        def porownaj_jeden_parametr(atrybut, wartosc_z_crossref, atrybut_label=None):
            rezultat = Komparator.porownaj(atrybut, wartosc_z_crossref)

            if "ORCID" in wartosc_z_crossref:
                wartosc_z_crossref["ORCID"] = normalize_orcid(
                    wartosc_z_crossref["ORCID"]
                )
            dane_porownania.append(
                {
                    "atrybut": atrybut_label or atrybut,
                    "orig_atrybut": atrybut,
                    "wartosc_z_crossref_print": json_format_with_wrap(
                        wartosc_z_crossref
                    ),
                    "wartosc_z_crossref": wartosc_z_crossref,
                    "rezultat": rezultat,
                }
            )

        for atrybut in cls.atrybuty.do_zmatchowania:
            wartosc_z_crossref = json_data.get(atrybut)

            if wartosc_z_crossref is None or not wartosc_z_crossref:
                continue

            if isinstance(wartosc_z_crossref, (list, tuple)):
                if len(wartosc_z_crossref) == 0:
                    continue

                if len(wartosc_z_crossref) > 1:
                    for no, elem in enumerate(wartosc_z_crossref):
                        porownaj_jeden_parametr(
                            atrybut, elem, atrybut_label=f"{atrybut}.{no}"
                        )
                else:
                    porownaj_jeden_parametr(
                        atrybut,
                        wartosc_z_crossref[0],
                    )
            else:
                porownaj_jeden_parametr(atrybut, wartosc_z_crossref)

        return dane_porownania

    @classmethod
    def _dane(cls, json_data, atrybuty, key_in_atrybuty=True):
        return [
            (key, {"original": item, "print": json_format_with_wrap(item)})
            for key, item in sorted(json_data.items())
            if (key in atrybuty) == key_in_atrybuty
        ]

    @classmethod
    def dane_do_skopiowania(cls, json_data):
        return cls._dane(json_data, Komparator.atrybuty.do_skopiowania)

    @classmethod
    def dane_ignorowane(cls, json_data):
        return cls._dane(json_data, Komparator.atrybuty.ignorowane)

    @classmethod
    def dane_obce(cls, json_data):
        return cls._dane(
            json_data, Komparator.atrybuty.wszystkie, key_in_atrybuty=False
        )

    @classmethod
    def czy_rekord_ma_odpowiednik_w_bpp(cls, dane_porownania_dict):
        if dane_porownania_dict is None or not dane_porownania_dict:
            return

        for atrybut in Komparator.atrybuty.do_porownania_rekordu:
            wynik_porownania = dane_porownania_dict.get(atrybut)

            if wynik_porownania is None:
                continue

            if (
                wynik_porownania.get("rezultat")
                and wynik_porownania.get("rezultat").rekord_po_stronie_bpp
            ):
                return wynik_porownania.get("rezultat").rekord_po_stronie_bpp



# -*- encoding: utf-8 -*-


DANE_OPEN_ACCESS = """Wersja_Tekstu_OpenAccess	ORIGINAL_AUTHOR	Oryginalna wersja autorska
Wersja_Tekstu_OpenAccess	FINAL_AUTHOR	Ostateczna wersja autorska
Wersja_Tekstu_OpenAccess	FINAL_PUBLISHED	Ostateczna wersja opublikowana
Licencja_OpenAccess	CC-BY	Creative Commons - Uznanie Autorstwa (CC-BY)
Licencja_OpenAccess	CC-BY-SA	Creative Commons - Uznanie Autorstwa - Na Tych Samych Warunkach (CC-BY-SA)
Licencja_OpenAccess	CC-BY-NC	Creative Commons - Uznanie Autorstwa - Użycie niekomercyjne (CC-BY-NC);
Licencja_OpenAccess	CC-BY-ND	Creative Commons - Uznanie Autorstwa - Bez utworów zależnych (CC-BY-ND)
Licencja_OpenAccess	CC-BY-NC-SA	Creative Commons - Uznanie Autorstwa - Użycie niekomercyjne - Na tych samych warunkach (CC-BY-NC-SA)
Licencja_OpenAccess	CC-BY-NC-ND	Creative Commons - Uznanie Autorstwa - Użycie niekomercyjne - Bez utworów zależnych (CC-BY-NC-ND)
Licencja_OpenAccess	OTHER	inna otwarta licencja
Czas_Udostepnienia_OpenAccess	BEFORE_PUBLICATION	przed opublikowaniem
Czas_Udostepnienia_OpenAccess	AT_PUBLICATION	w momencie opublikowania
Czas_Udostepnienia_OpenAccess	AFTER_PUBLICATION	po opublikowaniu
Tryb_OpenAccess_Wydawnictwo_Ciagle	OPEN_JOURNAL	Otwarte czasopismo
Tryb_OpenAccess_Wydawnictwo_Ciagle	OPEN_REPOSITORY	Otwarte repositorium
Tryb_OpenAccess_Wydawnictwo_Ciagle	OTHER	Inne
Tryb_OpenAccess_Wydawnictwo_Zwarte	PUBLISHER_WEBSITE	Witryna wydawcy
Tryb_OpenAccess_Wydawnictwo_Zwarte	OPEN_REPOSITORY	Otwarte repositorium
Tryb_OpenAccess_Wydawnictwo_Zwarte	OTHER	Inne"""

def get_openaccess_data():
    for model_name, skrot, nazwa in [x.split('\t') for x in DANE_OPEN_ACCESS.split("\n")]:
        yield model_name.lower(), skrot, nazwa



from django.db import models
from model_utils import Choices

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from bpp.const import TRYB_DOSTEPU


class Element_Repozytorium(models.Model):
    ER_TRYB_DOSTEPU = Choices(
        (TRYB_DOSTEPU.NIEJAWNY, "niejawny"),
        (TRYB_DOSTEPU.TYLKO_W_SIECI, "tylko w sieci"),
        (TRYB_DOSTEPU.JAWNY, "jawny"),
    )

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()

    rekord = GenericForeignKey("content_type", "object_id")

    rodzaj = models.CharField(max_length=200)
    nazwa_pliku = models.CharField(max_length=200)
    tryb_dostepu = models.PositiveSmallIntegerField(choices=ER_TRYB_DOSTEPU)

    class Meta:
        verbose_name = "element repozytorium"
        verbose_name_plural = "elementy repozytorium"



# Generated by Django 3.0.14 on 2021-08-07 07:14

# Generated by Django 3.0.14 on 2021-08-05 21:06
import warnings

from django.db import migrations

from bpp.util import pbar


def value(elem, *path, return_none=False):
    v = None
    if elem.versions:
        for _elem in elem.versions:
            if _elem["current"]:
                v = _elem
                break

    # v = elem.current_version

    if v is None:
        warnings.warn(
            f"Model {elem.__class__} with id {elem.mongoId} has NO current_version!"
        )
        if return_none:
            return
        return "[brak current_version]"

    for elem in path:
        if elem in v:
            v = v[elem]
        else:
            if return_none:
                return None
            return f"[brak {elem}]"
    return v


def value_or_none(elem, *path):
    return value(elem, *path, return_none=True)


MAX_TEXT_FIELD_LENGTH = 512


def _pull_up_on_save(elem, pull_up_on_save):
    for attr in pull_up_on_save:
        v = value_or_none(elem, "object", attr)
        if v is not None:
            if isinstance(v, str):
                if len(v) >= MAX_TEXT_FIELD_LENGTH:
                    v = v[:MAX_TEXT_FIELD_LENGTH]
        setattr(elem, attr, v)


def rebuild_table(model, puos):
    for elem in pbar(model.objects.all(), label=f"{model}..."):
        _pull_up_on_save(elem, puos)
        elem.save(update_fields=puos)


def rebuild(apps, schema_editor):
    for model, puos in [
        (
            apps.get_model("pbn_api", "Institution"),
            [
                "name",
                "addressCity",
                "addressStreet",
                "addressStreetNumber",
                "addressPostalCode",
                "polonUid",
            ],
        ),
        (
            apps.get_model("pbn_api", "Journal"),
            ["title", "websiteLink", "issn", "eissn", "mniswId"],
        ),
        (
            apps.get_model("pbn_api", "Publisher"),
            ["publisherName", "mniswId"],
        ),
        (
            apps.get_model("pbn_api", "Scientist"),
            ["lastName", "name", "qualifications", "orcid", "polonUid"],
        ),
        (
            apps.get_model("pbn_api", "Publication"),
            ["title", "doi", "publicUri", "isbn", "year"],
        ),
    ]:
        rebuild_table(model, puos)


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0019_auto_20210805_2306"),
    ]

    operations = [
        migrations.RunPython(rebuild, migrations.RunPython.noop),
    ]



import pytest


@pytest.mark.django_db
@pytest.mark.parametrize(
    "informacje,expected",
    [("bez sensu", None), ("2016 vol. 5 nr 10", "5"), ("2019 Bd 4 nr. 3 test", "4")],
)
def test_eksport_pbn_get_volume(informacje, expected, wydawnictwo_ciagle):
    wydawnictwo_ciagle.informacje = informacje
    assert wydawnictwo_ciagle.numer_tomu() == expected


@pytest.mark.django_db
def test_eksport_pbn_volume(wydawnictwo_ciagle):
    wydawnictwo_ciagle.informacje = "bez sensu"
    assert wydawnictwo_ciagle.numer_tomu() is None

    wydawnictwo_ciagle.informacje = "2016 vol 4"
    assert wydawnictwo_ciagle.numer_tomu() == "4"


@pytest.mark.django_db
@pytest.mark.parametrize(
    "informacje,expected",
    [
        ("bez sensu", None),
        ("2016 vol. 5 nr 10", "10"),
        ("2019 Bd 4 nr. 3 test", "3 test"),
        ("2019 Bd 4 nr 3 test", "3 test"),
        ("2019 Bd 4 z. 3 test", "3 test"),
        ("2019 Bd 4 h. 3 test", "3 test"),
        ("2019 Bd 4 iss. 311 test", "311 test"),
    ],
)
def test_eksport_pbn_get_issue(informacje, expected, wydawnictwo_ciagle):
    wydawnictwo_ciagle.informacje = informacje
    assert wydawnictwo_ciagle.numer_wydania() == expected


@pytest.mark.django_db
def test_punktacja_zrodla(wydawnictwo_ciagle):
    assert wydawnictwo_ciagle.punktacja_zrodla() is None

    z = wydawnictwo_ciagle.zrodlo
    z.punktacja_zrodla_set.create(rok=1990, impact_factor=100)

    assert wydawnictwo_ciagle.punktacja_zrodla() is None

    z.punktacja_zrodla_set.create(rok=wydawnictwo_ciagle.rok, impact_factor=37)
    assert wydawnictwo_ciagle.punktacja_zrodla().impact_factor == 37



from pathlib import Path

import pytest
from model_bakery import baker

from fixtures.conftest import (
    NORMAL_DJANGO_USER_LOGIN,
    NORMAL_DJANGO_USER_PASSWORD,
    _webtest_login,
)
from import_dyscyplin.models import Import_Dyscyplin

from django.contrib.auth.models import Group

from bpp.const import GR_WPROWADZANIE_DANYCH


@pytest.fixture
def wprowadzanie_danych_user(normal_django_user):
    grp = Group.objects.get_or_create(name=GR_WPROWADZANIE_DANYCH)[0]
    normal_django_user.groups.add(grp)
    return normal_django_user


@pytest.fixture
def wd_app(webtest_app, wprowadzanie_danych_user):
    return _webtest_login(
        webtest_app, NORMAL_DJANGO_USER_LOGIN, NORMAL_DJANGO_USER_PASSWORD
    )


@pytest.fixture
def parent_path():
    return Path(__file__).parent


@pytest.fixture
def test1_xlsx(parent_path):
    return str(parent_path / "test1.xlsx")


@pytest.fixture
def default_xlsx(parent_path):
    return str(
        parent_path / ".." / "static" / "import_dyscyplin" / "xlsx" / "default.xlsx"
    )


@pytest.fixture
def test2_bad_header_xlsx(parent_path):
    return str(parent_path / "test2_bad_header.xlsx")


@pytest.fixture
def test3_multiple_sheets_xlsx(parent_path):
    return str(parent_path / "test3_multiple_sheets.xlsx")


@pytest.fixture
def test4_kasowanie_xlsx(parent_path):
    return str(parent_path / "test4_kasowanie.xlsx")


@pytest.fixture
def test5_kasowanie_subdyscypliny(parent_path):
    return str(parent_path / "test5_kasowanie_subdyscypliny.xlsx")


@pytest.fixture
def conftest_py(parent_path):
    return str(parent_path / "conftest.py")


@pytest.fixture
def import_dyscyplin(db, rok):
    return baker.make(Import_Dyscyplin, rok=rok)



from __future__ import annotations

from import_common.normalization import normalize_isbn, normalize_issn
from ..exceptions import (
    CharakterFormalnyMissingPBNUID,
    DOIorWWWMissing,
    LanguageMissingPBNUID,
    PKZeroExportDisabled,
    StatementsMissing,
)
from .autor import AutorSimplePBNAdapter, AutorZDyscyplinaPBNAdapter
from .wydawca import WydawcaPBNAdapter
from .wydawnictwo_autor import WydawnictwoAutorToStatementPBNAdapter
from .wydawnictwo_nadrzedne import WydawnictwoNadrzednePBNAdapter
from .zrodlo import ZrodloPBNAdapter

from django.utils.functional import cached_property

from bpp import const
from bpp.const import TO_REDAKTOR, TO_REDAKTOR_TLUMACZENIA, TO_TLUMACZ
from bpp.models import BazaModeluOdpowiedzialnosciAutorow, Uczelnia
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.util import strip_html


class WydawnictwoPBNAdapter:
    CHAPTER = "CHAPTER"
    BOOK = "BOOK"
    ARTICLE = "ARTICLE"
    EDITED_BOOK = "EDITED_BOOK"

    export_pk_zero = True
    always_affiliate_to_uid = None

    def __init__(
        self,
        original: Wydawnictwo_Ciagle | Wydawnictwo_Zwarte,
        request=None,
        uczelnia=None,
        export_pk_zero=None,
        always_affiliate_to_uid=None,
    ):
        """
        :param always_affiliate_to_uid: jeżeli podany, użyj właśnie tego UID jako UID jako
        każdej afiliacji we własnej jednostce (czyli w nie-obcej i zatrudniającej pracowników
        i gdzie autor ma afiliację). W ten sposób każdy autor, który afiliuje na jednostkę
        uczelni i jest w jednostce która skupia pracowników będzie miał wysłaną afiliację na
        uczelnię -- niezależnie od wypełnienia uid jednostki lub nie.
        """
        self.original = original

        if export_pk_zero is not None:
            self.export_pk_zero = export_pk_zero
        else:
            if request is not None and uczelnia is None:
                uczelnia = Uczelnia.objects.get_for_request(request)

            if uczelnia is not None:
                if uczelnia.pbn_api_nie_wysylaj_prac_bez_pk:
                    self.export_pk_zero = False

        if always_affiliate_to_uid is not None:
            self.always_affiliate_to_uid = always_affiliate_to_uid

            class FakeJednostka:
                pbn_uid_id = self.always_affiliate_to_uid

            self.fake_jednostka = FakeJednostka()

    @cached_property
    def typy_ogolne_autorow(self):
        return set(
            self.original.autorzy_set.order_by()
            .values_list("typ_odpowiedzialnosci__typ_ogolny", flat=True)
            .distinct()
        )

    def pod_redakcja(self):
        # zwraca True jezeli self.original to praca 'pod redakcja' czyli ze ma tylko
        # redaktorow

        if (
            len(self.typy_ogolne_autorow) == 1
            and TO_REDAKTOR in self.typy_ogolne_autorow
        ):
            return True
        return False

    def get_translation(self):
        # Jeżeli praca ma wyłącznie tłumaczy lub redaktorów tłumaczy to jest tłumaczeniem

        lst = list(self.typy_ogolne_autorow)
        if TO_REDAKTOR_TLUMACZENIA in lst:
            lst.remove(TO_REDAKTOR_TLUMACZENIA)
        if TO_TLUMACZ in lst:
            lst.remove(TO_TLUMACZ)
        if not lst:
            return True

        # Praca ma jeszcze jakieś typu autorów więc potencjalnie może nie być tlumaczeniem
        return False

    def nr_tomu(self):
        if hasattr(self.original, "numer_tomu"):
            ntomu = self.original.numer_tomu()
            if ntomu is not None:
                return ntomu

        if hasattr(self.original, "tom"):
            if self.original.tom:
                return self.original.tom

    def get_type(self):
        if self.original.charakter_formalny.rodzaj_pbn == const.RODZAJ_PBN_ARTYKUL:
            return WydawnictwoPBNAdapter.ARTICLE
        elif self.original.charakter_formalny.rodzaj_pbn == const.RODZAJ_PBN_KSIAZKA:
            if self.pod_redakcja():
                return WydawnictwoPBNAdapter.EDITED_BOOK
            return WydawnictwoPBNAdapter.BOOK
        elif self.original.charakter_formalny.rodzaj_pbn == const.RODZAJ_PBN_ROZDZIAL:
            return WydawnictwoPBNAdapter.CHAPTER
        else:
            raise CharakterFormalnyMissingPBNUID(
                f"Rodzaj dla PBN nie określony dla charakteru formalnego {self.original.charakter_formalny}"
            )

    def pbn_get_json(self):
        ret = {
            "title": strip_html(self.original.tytul_oryginalny),
            "year": self.original.rok,
            "type": self.get_type(),
        }

        # "openAccess": {
        #     "releaseDate": "2021-05-19T00:56:06.872Z",
        #     "releaseDateMonth": "JANUARY",
        #     "releaseDateYear": 0,
        #   },

        oa = {}
        if self.original.openaccess_wersja_tekstu_id is not None:
            #     "textVersion": "ORIGINAL_AUTHOR"
            oa["textVersion"] = self.original.openaccess_wersja_tekstu.skrot
        if self.original.openaccess_licencja_id is not None:
            #     "license": "CC_BY",
            oa["license"] = self.original.openaccess_licencja.skrot.replace("-", "_")
        if self.original.openaccess_czas_publikacji_id is not None:
            # "releaseDateMode": "BEFORE_PUBLICATION",
            oa["releaseDateMode"] = self.original.openaccess_czas_publikacji.skrot

        if oa.get("releaseDateMode") == "AFTER_PUBLICATION":
            # https://pbn.nauka.gov.pl/centrum-pomocy/faq-kategoria/dodawanie-publikacji/
            # tylko w przypadku udostępnienia po opublikowaniu należy podać liczbę miesięcy
            # jakie upłynęły od dnia opublikowania do dnia udostępnienia publikacji w sposób otwarty
            if self.original.openaccess_ilosc_miesiecy is not None:
                #     "months": 0,
                oa["months"] = str(self.original.openaccess_ilosc_miesiecy)
            else:
                oa["months"] = "0"

        if self.original.openaccess_tryb_dostepu_id is not None:
            if ret["type"] == "ARTICLE":
                oa["modeArticle"] = self.original.openaccess_tryb_dostepu.skrot
            else:
                oa["modeMonograph"] = self.original.openaccess_tryb_dostepu.skrot
        if self.original.public_dostep_dnia is not None:
            oa["releaseDate"] = str(self.original.public_dostep_dnia)
        elif self.original.dostep_dnia is not None:
            oa["releaseDate"] = str(self.original.dostep_dnia)

        if oa.get("releaseDate") is None:
            oa["releaseDateMonth"] = "JANUARY"
            oa["releaseDateYear"] = str(self.original.rok)

        if (
            oa.get("license")
            and oa.get("textVersion")
            and oa.get("modeArticle")
            and oa.get("releaseDateMode")
        ):
            ret["openAccess"] = oa

        volume = self.nr_tomu()
        if volume:
            ret["volume"] = volume

        # Jeżeli pole "Strony" istnieje i ma jakąś wartość, to weź jego wartość
        if hasattr(self.original, "strony"):
            strony = self.original.strony
            if strony:
                ret["pagesFromTo"] = strony

        # Jeżeli pole "Strony" było puste a mamy funkcję "zakres_stron" i ono coś zwróci,
        # to weź tą wartość:
        if hasattr(self.original, "zakres_stron") and not ret.get("pagesFromTo"):
            zakres_stron = self.original.zakres_stron()
            if zakres_stron:
                ret["pagesFromTo"] = zakres_stron

        if self.original.doi:
            ret["doi"] = self.original.doi

        if self.original.jezyk.pbn_uid_id is None:
            raise LanguageMissingPBNUID(
                f'Język rekordu "{self.original.jezyk}" nie ma określonego odpowiednika w PBN'
            )

        ret["mainLanguage"] = self.original.jezyk.pbn_uid.code
        if self.original.jezyk_orig:

            if self.original.jezyk_orig.pbn_uid_id is None:
                raise LanguageMissingPBNUID(
                    f'Język *oryginalny* rekordu "{self.original.jezyk}" nie ma określonego odpowiednika w PBN'
                )

            ret["originalLanguage"] = self.original.jezyk_orig.pbn_uid.code

        if self.original.public_www:
            ret["publicUri"] = self.original.public_www
        elif self.original.www:
            ret["publicUri"] = self.original.www
        elif (
            ret["type"] == WydawnictwoPBNAdapter.CHAPTER
            and hasattr(self.original, "wydawnictwo_nadrzedne_id")
            and self.original.wydawnictwo_nadrzedne_id is not None
            and (
                self.original.wydawnictwo_nadrzedne.public_www
                or self.original.wydawnictwo_nadrzedne.www
            )
        ):
            # W sytuacji, gdy eksportujemy rozdział, jako adres WWW można spróbować użyć
            # adres WWW wydawnictwa nadrzędnego:
            ret["publicUri"] = (
                self.original.wydawnictwo_nadrzedne.public_www
                or self.original.wydawnictwo_nadrzedne.www
            )

        if not ret.get("doi") and not ret.get("publicUri"):
            raise DOIorWWWMissing("Musi być DOI lub adres WWW")

        if hasattr(self.original, "zrodlo"):
            ret["journal"] = ZrodloPBNAdapter(self.original.zrodlo).pbn_get_json()

        authors = []
        editors = []
        translators = []
        translationEditors = []
        statements = []
        jednostki = set()
        for elem in self.original.autorzy_set.all().select_related():
            elem: BazaModeluOdpowiedzialnosciAutorow
            #
            # Jeżeli dany rekord Wydawnictwo_..._Autor ma dyscyplinę, to takiego autora
            # eksportujemy 'w pełni' tzn ze wszystkimi posiadanymi przez niego identyfikatorami
            # typu PBN UID czy ORCID:
            #

            if elem.dyscyplina_naukowa_id is None or elem.przypieta is False:
                #
                # Autor nie ma dyscypliny lub ma odpiętą dyscyplinę -- prosty eksport w formacie imię + nazwisko
                #
                author = AutorSimplePBNAdapter(elem.autor).pbn_get_json()
            else:
                author = AutorZDyscyplinaPBNAdapter(elem.autor).pbn_get_json()

            jednostka = elem.jednostka
            if (
                elem.afiliuje
                and jednostka.pk != -1
                and elem.jednostka.skupia_pracownikow
            ):
                if self.always_affiliate_to_uid:
                    author["affiliations"] = [self.always_affiliate_to_uid]
                    jednostki.add(self.fake_jednostka)
                else:
                    if jednostka.pbn_uid_id is None:
                        # raise WillNotExportError(
                        #     f"Jednostka {jednostka} nie ma ustawionego odpowiednika w PBN"
                        # )
                        pass
                    else:
                        author["affiliations"] = [jednostka.pbn_uid_id]
                        jednostki.add(elem.jednostka)

                statement = WydawnictwoAutorToStatementPBNAdapter(elem).pbn_get_json()
                if statement:
                    statements.append(statement)

            if elem.typ_odpowiedzialnosci.typ_ogolny == const.TO_REDAKTOR:
                editors.append(author)
            elif elem.typ_odpowiedzialnosci.typ_ogolny == const.TO_TLUMACZ:
                translators.append(author)
            elif elem.typ_odpowiedzialnosci.typ_ogolny == const.TO_REDAKTOR_TLUMACZENIA:
                translationEditors.append(author)
            else:
                authors.append(author)

        if authors:
            ret["authors"] = authors
        if translators:
            ret["translators"] = translators
        if editors:
            ret["editors"] = editors
        if translationEditors:
            ret["translationEditors"] = translationEditors
        if statements:
            ret["statements"] = statements

        if hasattr(self.original, "isbn"):
            if self.original.isbn:
                ret["isbn"] = normalize_isbn(self.original.isbn)
            elif hasattr(self.original, "e_isbn"):
                # Jeżeli nie ma ISBN a jest wartość w polu E-ISBN, to użyj jej:
                if self.original.e_isbn:
                    ret["isbn"] = normalize_isbn(self.original.e_isbn)

        if hasattr(self.original, "issn"):
            if self.original.issn:
                ret["issn"] = normalize_issn(self.original.issn)

        if hasattr(self.original, "numer_wydania"):
            nr_wydania = self.original.numer_wydania()
            if nr_wydania:
                ret["issue"] = nr_wydania

        if hasattr(self.original, "seria_wydawnicza_id"):
            seria = self.original.seria_wydawnicza_id
            if seria is not None:
                ret["series"] = self.original.seria_wydawnicza.nazwa

        if hasattr(self.original, "numer_w_serii"):
            if self.original.numer_w_serii:
                ret["numberInSeries"] = self.original.numer_w_serii

        if self.original.pbn_uid_id is not None:
            ret["objectId"] = self.original.pbn_uid_id

        if hasattr(self.original, "miejsce_i_rok"):
            if self.original.miejsce_i_rok:
                miejsce = " ".join(self.original.miejsce_i_rok.split(" ")[:-1]).strip()
                if miejsce:
                    ret["publicationPlace"] = miejsce

        if hasattr(self.original, "wydawca"):
            if self.original.wydawca_id:
                ret["publisher"] = WydawcaPBNAdapter(
                    self.original.wydawca
                ).pbn_get_json()
            else:
                if self.original.wydawca_opis:
                    ret["publisher"] = {"name": self.original.wydawca_opis}

        ret["translation"] = self.get_translation()

        if hasattr(self.original, "wydawnictwo_nadrzedne_id"):
            if self.original.wydawnictwo_nadrzedne_id is not None:
                ret["book"] = WydawnictwoNadrzednePBNAdapter(
                    self.original.wydawnictwo_nadrzedne
                ).pbn_get_json()

        institutions = {}
        for jednostka in jednostki:
            institutions[jednostka.pbn_uid_id] = {
                "objectId": jednostka.pbn_uid_id,
                # "polonUuid": jednostka.pbn_uid.value("object", "polonUuid"),
                # "versionHash": jednostka.pbn_uid.value("object", "versionHash"),
            }
        if institutions:
            ret["institutions"] = institutions

        if (
            hasattr(self.original, "slowa_kluczowe")
            and self.original.slowa_kluczowe.exists()
        ):
            if "languageData" not in ret:
                ret["languageData"] = {}

            slowa_kluczowe = list(
                self.original.slowa_kluczowe.all().values_list("name", flat=True)
            )
            # Zakładamy, że wszystkie słowa kluczowe są w języku rekordu nadrzędnego
            ret["languageData"]["keywords"] = [
                {"keywords": slowa_kluczowe, "lang": ret["mainLanguage"]}
            ]

        if hasattr(self.original, "streszczenia"):
            if "languageData" not in ret:
                ret["languageData"] = {}

            if self.original.streszczenia.exists():
                ret["languageData"]["abstracts"] = []

                for streszczenie in self.original.streszczenia.all():
                    if streszczenie.jezyk_streszczenia_id is None:
                        streszczenie.jezyk_streszczenia = self.original.jezyk

                    if (
                        streszczenie.streszczenie is None
                        or not streszczenie.streszczenie.strip()
                    ):
                        continue

                    ret["languageData"]["abstracts"].append(
                        {
                            "lang": streszczenie.jezyk_streszczenia.pbn_uid.code,
                            "text": streszczenie.streszczenie,
                        }
                    )

        if hasattr(self.original, "opl_pub_amount") and hasattr(
            self.original, "opl_pub_cost_free"
        ):
            # ModelZOplataZaPublikacje
            fee = {}

            if self.original.opl_pub_cost_free is True:
                fee["amount"] = 0
                fee["costFreePublication"] = True
                fee["other"] = False
                fee["researchOrDevelopmentProjectsFinancialResources"] = False
                fee["researchPotentialFinancialResources"] = False

            if (
                self.original.opl_pub_amount is not None
                and self.original.opl_pub_amount > 0
            ):
                fee["amount"] = str(self.original.opl_pub_amount)
                fee["costFreePublication"] = False
                fee["other"] = self.original.opl_pub_other or False
                fee["researchOrDevelopmentProjectsFinancialResources"] = (
                    self.original.opl_pub_research_or_development_projects or False
                )
                fee["researchPotentialFinancialResources"] = (
                    self.original.opl_pub_research_potential or False
                )

            if fee:
                ret["fee"] = fee

        if ret["type"] in [
            WydawnictwoPBNAdapter.ARTICLE,
            WydawnictwoPBNAdapter.CHAPTER,
        ]:

            if self.export_pk_zero is False:
                if hasattr(self.original, "punkty_kbn"):
                    if self.original.punkty_kbn == 0:
                        raise PKZeroExportDisabled(
                            "Eksport prac typu artykuł i typu rozdział z PK równym zero jest wyłączony w konfiguracji "
                            "systemu (obiekt Uczelnia). "
                        )

            if not ret.get("statements"):
                raise StatementsMissing(
                    "Nie wyślę rekordu artykułu lub rozdziału bez zadeklarowanych oświadczeń autorów (dyscyplin). "
                )

        return ret



from channels.auth import AuthMiddlewareStack
from channels.routing import ProtocolTypeRouter, URLRouter, ChannelNameRouter
from notifications import routing as notifications_routing

application = ProtocolTypeRouter(
    {
        # (http->django views is added by default)
        "websocket": AuthMiddlewareStack(
            URLRouter(notifications_routing.websocket_urlpatterns)
        ),
    }
)



# Generated by Django 3.0.14 on 2021-05-09 21:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0010_auto_20210504_0040"),
    ]

    operations = [
        migrations.AddField(
            model_name="sentdata",
            name="exception",
            field=models.TextField(blank=True, max_length=65535, null=True),
        ),
        migrations.AddField(
            model_name="sentdata",
            name="uploaded_okay",
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0061_utworz_jednostka_wydzial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='jednostka_wydzial',
            options={'ordering': ('-od',), 'verbose_name': 'powi\u0105zanie jednostka-wydzia\u0142', 'verbose_name_plural': 'powi\u0105zania jednostka-wydzia\u0142'},
        ),
    ]



import openpyxl
from django.core.exceptions import ValidationError
from django.db import models
from openpyxl.utils.exceptions import InvalidFileException

from ewaluacja2021.util import find_header_row


def validate_xlsx(obj: models.FileField):
    try:
        return openpyxl.load_workbook(obj)
    except InvalidFileException as e:
        raise ValidationError(f"Nieobsługiwany rodzaj pliku ({e}) ")
    except BaseException as e:
        raise ValidationError(f"Błąd przy próbie otwarcia pliku ({e})")


class xlsx_header_validator:
    """Upewnia się, że plik XLS ma zadany nagłówek.

    Dodatkowo, ten validator zapewnia również walidację typu pliku XLSX, ponieważ korzysta
    z funkcji ``validate_xls``"""

    def __init__(self, columns, max_header_row=100):
        """
        @param columns: kolumny w takiej kolejności, jaka powinna być w pliku XLS,
        może to być jedynie ich część (nie wszystkie), ale muszą być od początku;
        wielkość liter nie ma znaczenia, polskie litery mogą być usuwane.

        @param max_header_row: szukaj wiersza nagłówka maksymalnie do ``max_header_row``
        wierszy
        """
        self.cols = columns
        self.max_header_row = max_header_row

    def deconstruct(self):
        return (
            "ewaluacja2021.validators.xlsx_header_validator",
            (),
            {"columns": self.cols, "max_header_row": self.max_header_row},
        )

    def __eq__(self, other):
        return other.cols == self.cols and other.max_header_row == self.max_header_row

    def __call__(self, obj: models.FileField):
        wb = validate_xlsx(obj)
        header_row = find_header_row(
            wb.worksheets[0], header_row=self.cols, max_header_row=self.max_header_row
        )

        if header_row is not None:
            return True

        quoted_cols = [f'"{x}"' for x in self.cols]
        raise ValidationError(
            f"Nie znaleziono wiersza nagłówka wśród pierwszych {self.max_header_row} wierszy pierwszego arkusza "
            f"skoroszytu. Oczekiwany wiersz nagłówka powinien zaczynać się od pierwszej kolumny (kolumna 'A') "
            f"i zawierać następujące kolumny, w takiej kolejności, tak zatytułowane, bez przerw pomiędzy: "
            f"{', '.join(quoted_cols)}. "
        )



from django.db import models
from django.db.models import JSONField


class Log(models.Model):
    started_on = models.DateTimeField(auto_now_add=True)
    finished_on = models.DateTimeField(blank=True, null=True)
    finished_successfully = models.BooleanField(null=True, blank=True, default=None)

    command_name = models.TextField()
    args = JSONField(blank=True, null=True)

    stdout = models.TextField(blank=True, null=True)
    stderr = models.TextField(blank=True, null=True)
    traceback = models.TextField(blank=True, null=True)

    def save(
        self, force_insert=False, force_update=False, using=None, update_fields=None
    ):
        return super().save(
            force_insert=force_insert,
            force_update=force_update,
            using=using,
            update_fields=update_fields,
        )

    def __str__(self):
        return f'Results of command "{self.command_name}" ran on {self.started_on}'



import pytest
from model_bakery import baker

from crossref_bpp.utils import json_format_with_wrap, perform_trigram_search
from import_common.core import normalized_db_title

from bpp.models import Wydawnictwo_Ciagle


def test_json_format_with_wrap():
    x = "x" * 90
    res = json_format_with_wrap(x)
    assert len(res.split("\n")[0]) == 70


@pytest.mark.django_db
def test_perform_trigram_search():
    wc = baker.make(
        Wydawnictwo_Ciagle, tytul_oryginalny="Ta nazwa będzie miała polskie litery."
    )

    baker.make(
        Wydawnictwo_Ciagle,
        tytul_oryginalny="Ta nazwa będzie miała polskie litery i może coś jeszcze.",
    )
    baker.make(
        Wydawnictwo_Ciagle,
        tytul_oryginalny="Początek tytułu. Ta nazwa będzie miała polskie litery i może coś jeszcze.",
    )
    baker.make(Wydawnictwo_Ciagle, tytul_oryginalny="Kto to wie co to będzie.")
    baker.make(
        Wydawnictwo_Ciagle, tytul_oryginalny="Ta nazwa bedzie miala waskie znaki."
    )
    baker.make(
        Wydawnictwo_Ciagle, tytul_oryginalny="Ta nazwa bedzie miala proste litery."
    )

    ret = perform_trigram_search(
        Wydawnictwo_Ciagle.objects.all(),
        trigram_db_field=normalized_db_title,
        trigram_db_value="Ta nazwa bedzie miala polskie litery",
    )

    assert len(ret) == 2
    assert wc in ret



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-29 10:05
from __future__ import unicode_literals

import bpp.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0105_rekord_z_konferencja'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_ranking_autorow',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj ranking autorów'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_autorow',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj raport autorów'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_dla_komisji_centralnej',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj raport dla Komisji Centralnej'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_jednostek',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj ranking jednostek'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_raport_wydzialow',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj ranking wydziałów'),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='pokazuj_status_korekty',
            field=bpp.models.fields.OpcjaWyswietlaniaField(choices=[('always', 'zawsze'), ('logged-in', 'tylko dla zalogowanych'), ('never', 'nigdy')], default='always', max_length=50, verbose_name='Pokazuj status korekty na stronie rekordu'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-06-24 16:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0140_auto_20180624_1620'),
    ]

    operations = [
        migrations.AddField(
            model_name='konferencja',
            name='typ_konferencji',
            field=models.SmallIntegerField(blank=True, choices=[(1, '🚆 krajowa'), (2, '✈️ międzynarodowa'), (3, '🚲 lokalna')], null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-24 17:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0156_auto_20190324_1828'),
    ]

    operations = [
        migrations.RenameField(
            model_name='autor_dyscyplina',
            old_name='dyscyplina',
            new_name='dyscyplina_naukowa',
        ),
        migrations.RenameField(
            model_name='autor_dyscyplina',
            old_name='subdyscyplina',
            new_name='subdyscyplina_naukowa',
        ),
    ]



import sys

import django
from django.core.management import call_command

from pbn_api.integrator.odswiez_tabele_publikacji import odswiez_tabele_publikacji
from pbn_api.integrator.pobierz_skasowane_prace import pobierz_skasowane_prace

django.setup()


from pbn_api import integrator
from pbn_api.exceptions import IntegracjaWylaczonaException
from pbn_api.integrator import (
    integruj_autorow_z_uczelni,
    integruj_instytucje,
    integruj_jezyki,
    integruj_kraje,
    integruj_oswiadczenia_z_instytucji,
    integruj_publikacje_instytucji,
    integruj_uczelnie,
    integruj_wszystkich_niezintegrowanych_autorow,
    integruj_wydawcow,
    integruj_zrodla,
    pobierz_instytucje,
    pobierz_konferencje,
    pobierz_ludzi_z_uczelni,
    pobierz_oswiadczenia_z_instytucji,
    pobierz_prace_po_doi,
    pobierz_prace_po_isbn,
    pobierz_publikacje_z_instytucji,
    pobierz_rekordy_publikacji_instytucji,
    pobierz_wydawcow_mnisw,
    pobierz_wydawcow_wszystkich,
    pobierz_zrodla,
    sprawdz_ilosc_autorow_przy_zmatchowaniu,
    synchronizuj_publikacje,
    usun_wszystkie_oswiadczenia,
    usun_zerowe_oswiadczenia,
    weryfikuj_orcidy,
    wyswietl_niezmatchowane_ze_zblizonymi_tytulami,
)
from pbn_api.management.commands.util import PBNBaseCommand

from bpp.models import Uczelnia


def check_end_before(stage, end_before_stage):
    if end_before_stage == stage:
        sys.exit(0)


class Command(PBNBaseCommand):
    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            "--disable-multiprocessing", action="store_true", default=False
        ),

        parser.add_argument("--start-from-stage", type=int, default=0)
        parser.add_argument("--end-before-stage", type=int, default=None)
        parser.add_argument("--just-one-stage", action="store_true"),

        parser.add_argument("--clear-all", action="store_true", default=False)
        parser.add_argument("--clear-publications", action="store_true", default=False)
        parser.add_argument(
            "--clear-match-publications", action="store_true", default=False
        )
        parser.add_argument("--enable-all", action="store_true", default=False)
        parser.add_argument("--enable-delete-all", action="store_true", default=False)
        parser.add_argument("--enable-delete-zeros", action="store_true", default=False)

        parser.add_argument("--enable-system-data", action="store_true", default=False)
        parser.add_argument(
            "--enable-pobierz-zrodla", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-integruj-zrodla", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-download-people-institution", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-download-people-all", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-integrate-people-institution", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-integrate-people-all", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-check-orcid-people", action="store_true", default=False
        )
        parser.add_argument("--enable-publishers", action="store_true", default=False)
        parser.add_argument("--enable-conferences", action="store_true", default=False)
        parser.add_argument("--enable-institutions", action="store_true", default=False)
        parser.add_argument(
            "--enable-pobierz-po-doi", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-pobierz-po-isbn", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-pobierz-wszystkie-publikacje", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-pobierz-publikacje-instytucji", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-pobierz-rekordy-publikacji-instytucji",
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--enable-pobierz-oswiadczenia-instytucji",
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--enable-odswiez-tabele-publikacji",
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--enable-integruj-wszystkie-publikacje", action="store_true", default=False
        )
        parser.add_argument(
            "--enable-integruj-publikacje-instytucji",
            action="store_true",
            default=False,
        )
        parser.add_argument("--skip-pages", type=int, default=0)
        parser.add_argument("--enable-sync", action="store_true", default=False)
        parser.add_argument("--force-upload", action="store_true", default=False)
        parser.add_argument("--only-bad", action="store_true", default=False)
        parser.add_argument("--only-new", action="store_true", default=False)
        parser.add_argument(
            "--delete-statements-before-upload", action="store_true", default=None
        )
        parser.add_argument(
            "--export-pk-zero",
            action="store_true",
            default=None,
        )
        parser.add_argument(
            "--disable-progress-bar", action="store_true", default=False
        )

    def handle(
        self,
        app_id,
        app_token,
        base_url,
        user_token,
        start_from_stage,
        end_before_stage,
        just_one_stage,
        disable_multiprocessing,
        clear_all,
        clear_publications,
        clear_match_publications,
        enable_all,
        enable_delete_all,
        enable_delete_zeros,
        enable_system_data,
        enable_pobierz_zrodla,
        enable_integruj_zrodla,
        enable_download_people_institution,
        enable_download_people_all,
        enable_integrate_people_institution,
        enable_integrate_people_all,
        enable_check_orcid_people,
        enable_publishers,
        enable_conferences,
        enable_institutions,
        enable_pobierz_po_doi,
        enable_pobierz_po_isbn,
        enable_pobierz_wszystkie_publikacje,
        enable_pobierz_publikacje_instytucji,
        enable_pobierz_oswiadczenia_instytucji,
        enable_odswiez_tabele_publikacji,
        enable_pobierz_rekordy_publikacji_instytucji,
        enable_integruj_wszystkie_publikacje,
        enable_integruj_publikacje_instytucji,
        skip_pages,
        enable_sync,
        force_upload,
        only_bad,
        only_new,
        disable_progress_bar,
        delete_statements_before_upload,
        export_pk_zero,
        *args,
        **options
    ):
        if disable_multiprocessing:
            integrator.CPU_COUNT = "single"

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia is not None:
            if not uczelnia.pbn_integracja:
                raise IntegracjaWylaczonaException()
        client = self.get_client(app_id, app_token, base_url, user_token)

        if clear_all:
            integrator.clear_all()
            sys.exit(0)

        if clear_match_publications:
            integrator.clear_match_publications()
            sys.exit(0)

        if clear_publications:
            integrator.clear_publications()
            sys.exit(0)

        if just_one_stage:
            end_before_stage = start_from_stage + 1

        stage = 0
        if (enable_system_data or enable_all) and start_from_stage <= stage:
            integruj_jezyki(client)
            integruj_kraje(client)

        stage = 1
        check_end_before(stage, end_before_stage)
        if (enable_pobierz_zrodla or enable_all) and start_from_stage <= stage:
            pobierz_zrodla(client)

        stage = 2
        check_end_before(stage, end_before_stage)
        if (enable_integruj_zrodla or enable_all) and start_from_stage <= stage:
            integruj_zrodla(disable_progress_bar)

        stage = 3
        check_end_before(stage, end_before_stage)
        if (enable_institutions or enable_all) and start_from_stage <= stage:
            # Pobieranie instytucji musi odbywac się przed pobieraniem ludzi
            pobierz_instytucje(client)
            integruj_uczelnie()
            integruj_instytucje()

        # stage = 4
        # check_end_before(stage, end_before_stage)
        # if (enable_download_people_all or enable_all) and start_from_stage <= stage:
        #     os.makedirs("pbn_json_data", exist_ok=True)
        #     pobierz_ludzi_offline(client)
        #
        # stage = 5
        # check_end_before(stage, end_before_stage)
        # if (enable_download_people_all or enable_all) and start_from_stage <= stage:
        #     wgraj_ludzi_z_offline_do_bazy()

        stage = 6
        check_end_before(stage, end_before_stage)

        if (
            enable_download_people_institution or enable_all
        ) and start_from_stage <= stage:
            pobierz_ludzi_z_uczelni(client, Uczelnia.objects.default.pbn_uid_id)
        stage = 7
        check_end_before(stage, end_before_stage)

        if (
            enable_integrate_people_institution or enable_all
        ) and start_from_stage <= stage:
            integruj_autorow_z_uczelni(client, Uczelnia.objects.default.pbn_uid_id)
        stage = 8

        if (enable_integrate_people_all or enable_all) and start_from_stage <= stage:
            integruj_wszystkich_niezintegrowanych_autorow()
        stage = 9

        if (enable_check_orcid_people or enable_all) and start_from_stage <= stage:
            weryfikuj_orcidy(client, Uczelnia.objects.default.pbn_uid_id)
        stage = 10
        check_end_before(stage, end_before_stage)

        if (enable_publishers or enable_all) and start_from_stage <= stage:
            pobierz_wydawcow_wszystkich(client)
            pobierz_wydawcow_mnisw(client)
            integruj_wydawcow()
            call_command("pbn_importuj_wydawcow")
            # zamapuj_wydawcow nie trzeba, bo zostanie wywołany przez pbn_importuj_wydawców gdyby coś
            # call_command("zamapuj_wydawcow")

        stage = 11
        check_end_before(stage, end_before_stage)

        if (enable_conferences or enable_all) and start_from_stage <= stage:
            pobierz_konferencje(client)

        stage = 12
        check_end_before(stage, end_before_stage)

        #
        # Pobieranie wszystkich publikacji z całego PBNu - bez wiekszego sensu
        # do obecnych zastosowań
        #
        # if (
        #     enable_pobierz_wszystkie_publikacje
        # ) and start_from_stage <= stage:
        #     os.makedirs("pbn_json_data", exist_ok=True)
        #     pobierz_prace_offline(client)
        #
        # stage = 13
        # check_end_before(stage, end_before_stage)
        #
        # Wgrywanie wszystkich prac z offline do bazy
        #
        # if (
        #     enable_pobierz_wszystkie_publikacje
        # ) and start_from_stage <= stage:
        #     wgraj_prace_z_offline_do_bazy()
        #

        #
        # Pobieranie oswiadczen i publikacji z insytucji
        #

        if (
            enable_pobierz_rekordy_publikacji_instytucji or enable_all
        ) and start_from_stage <= stage:
            pobierz_rekordy_publikacji_instytucji(client)

        stage = 13
        check_end_before(stage, end_before_stage)
        if (
            enable_pobierz_publikacje_instytucji or enable_all
        ) and start_from_stage <= stage:
            pobierz_publikacje_z_instytucji(client)

        stage = 14
        check_end_before(stage, end_before_stage)

        if (
            enable_pobierz_oswiadczenia_instytucji or enable_all
        ) and start_from_stage <= stage:
            pobierz_oswiadczenia_z_instytucji(client)

        stage = 15

        if (
            enable_odswiez_tabele_publikacji or enable_all
        ) and start_from_stage <= stage:
            pobierz_skasowane_prace(client)

        stage = 16
        check_end_before(stage, end_before_stage)

        if (
            enable_odswiez_tabele_publikacji or enable_all
        ) and start_from_stage <= stage:
            odswiez_tabele_publikacji(client)

        stage = 17
        check_end_before(stage, end_before_stage)

        # if (enable_integruj_wszystkie_publikacje) and start_from_stage <= stage:
        #     integruj_wszystkie_publikacje(
        #         disable_multiprocessing, skip_pages=skip_pages
        #     )

        if (
            enable_integruj_publikacje_instytucji or enable_all
        ) and start_from_stage <= stage:
            integruj_publikacje_instytucji(
                disable_multiprocessing, skip_pages=skip_pages
            )

        stage = 18
        check_end_before(stage, end_before_stage)

        if (
            enable_pobierz_oswiadczenia_instytucji or enable_all
        ) and start_from_stage <= stage:
            integruj_oswiadczenia_z_instytucji()

        stage = 19
        check_end_before(stage, end_before_stage)

        if (enable_pobierz_po_doi or enable_all) and start_from_stage <= stage:
            pobierz_prace_po_doi(client)

        stage = 20
        check_end_before(stage, end_before_stage)

        if (enable_pobierz_po_isbn or enable_all) and start_from_stage <= stage:
            pobierz_prace_po_isbn(client)

        stage = 21
        check_end_before(stage, end_before_stage)

        if (
            enable_integruj_wszystkie_publikacje or enable_all
        ) and start_from_stage <= stage:
            wyswietl_niezmatchowane_ze_zblizonymi_tytulami()
            sprawdz_ilosc_autorow_przy_zmatchowaniu()

        stage = 22
        check_end_before(stage, end_before_stage)

        if enable_delete_all:
            usun_wszystkie_oswiadczenia(client)

        if enable_delete_zeros:
            usun_zerowe_oswiadczenia(client)

        if enable_sync:

            uczelnia = Uczelnia.objects.get_default()

            if export_pk_zero is None:
                export_pk_zero = not uczelnia.pbn_api_nie_wysylaj_prac_bez_pk

            if delete_statements_before_upload is None:
                delete_statements_before_upload = uczelnia.pbn_api_kasuj_przed_wysylka

            synchronizuj_publikacje(
                client=client,
                force_upload=force_upload,
                only_bad=only_bad,
                only_new=only_new,
                delete_statements_before_upload=delete_statements_before_upload,
                export_pk_zero=export_pk_zero,
            )



import pytest
from denorm.models import DirtyInstance
from django.core.exceptions import ValidationError
from django.db.models import ProtectedError
from model_bakery import baker

from bpp.models import Wydawca
from bpp.models.wydawca import Poziom_Wydawcy


@pytest.mark.django_db
def test_wydawnictwo_zwarte_wydawca_delete(wydawnictwo_zwarte, wydawca, denorms):
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.save()

    denorms.flush()
    assert DirtyInstance.objects.count() == 0

    with pytest.raises(ProtectedError):
        wydawca.delete()

    wydawnictwo_zwarte.wydawca = None
    wydawnictwo_zwarte.save()

    wydawca.delete()

    assert DirtyInstance.objects.count() == 1
    denorms.flush()


@pytest.mark.django_db
def test_wydawnictwo_zwarte_wydawca_change_nazwa(wydawnictwo_zwarte, wydawca, denorms):
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.save()

    denorms.flush()
    assert DirtyInstance.objects.all().count() == 0

    wydawca.nazwa = wydawca.nazwa + "X"
    wydawca.save()

    assert DirtyInstance.objects.all().count() == 2


@pytest.mark.django_db
def test_wydawnictwo_zwarte_wydawca_change_alias_dla(
    wydawnictwo_zwarte, wydawca, denorms
):
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.save()

    denorms.flush()
    assert DirtyInstance.objects.all().count() == 0

    wydawca2 = baker.make(Wydawca)

    wydawca.alias_dla = wydawca2
    wydawca.save()

    assert DirtyInstance.objects.all().count() == 5


@pytest.mark.django_db
def test_wydawnictwo_zwarte_wydawca_change_poziom_ten_sam_rok(
    wydawnictwo_zwarte, wydawca, rok, denorms
):
    wydawnictwo_zwarte.wydawca = wydawca
    wydawnictwo_zwarte.rok = rok
    wydawnictwo_zwarte.save()

    denorms.flush()
    assert DirtyInstance.objects.all().count() == 0

    pw = wydawca.poziom_wydawcy_set.create(rok=rok, poziom=1)

    assert DirtyInstance.objects.all().count() == 1
    denorms.flush()
    pw.poziom = 2
    pw.save()

    # Tu przebuduje wydawce + liste poziomow na wydawcy
    denorms.flush(run_once=True)

    # ... a teraz bedzie przebudowywał wyd. zwrate
    assert DirtyInstance.objects.all().count() == 1
    assert DirtyInstance.objects.first().object_id == wydawnictwo_zwarte.pk


@pytest.mark.django_db
def test_wydawca_get_tier(wydawca, rok):
    assert wydawca.get_tier(rok) == -1

    pw = wydawca.poziom_wydawcy_set.create(rok=rok, poziom=None)
    assert wydawca.get_tier(rok) is None

    pw.poziom = 1
    pw.save()
    assert wydawca.get_tier(rok) == 1


@pytest.mark.django_db
def test_wydawca_alias_get_tier(wydawca, alias_wydawcy, rok):
    wydawca.poziom_wydawcy_set.create(rok=rok, poziom=1)
    assert wydawca.get_tier(rok) == 1

    assert alias_wydawcy.get_tier(rok) == 1
    assert alias_wydawcy.get_tier(rok + 10) == -1


def test_wydawca_alias_nie_pozwol_stworzyc_poziomu_dla_aliasu(alias_wydawcy):
    with pytest.raises(ValidationError):
        alias_wydawcy.poziom_wydawcy_set.create(rok=2020, poziom=1)


def test_wydawca_alias_nie_pozwol_zrobic_aliasu_dla_posiadajacego_poziomy(wydawca):
    wydawca.poziom_wydawcy_set.create(rok=2020, poziom=2)
    w2 = baker.make(Wydawca)
    wydawca.alias_dla = w2
    with pytest.raises(ValidationError):
        wydawca.save()


def test_wydawca_alias_sam_do_siebie(wydawca):
    wydawca.alias_dla = wydawca
    with pytest.raises(ValidationError):
        wydawca.save()


def test_poziom_wydawcy_str(wydawca):
    pw = Poziom_Wydawcy.objects.create(wydawca=wydawca, rok=2020, poziom=1)

    assert str(pw) == 'Poziom wydawcy "Wydawca Testowy" za rok 2020'


@pytest.mark.django_db
def test_denorm_wydawca_ilosc_aliasow(denorms):
    w1 = Wydawca.objects.create(nazwa="123")
    assert w1.ile_aliasow == 0

    Wydawca.objects.create(nazwa="456", alias_dla=w1)

    denorms.flush()
    w1.refresh_from_db()
    assert w1.ile_aliasow == 1


@pytest.mark.django_db
def test_denorm_wydawca_poziomy_wydawcy(denorms):
    w1 = Wydawca.objects.create(nazwa="123")
    assert w1.lista_poziomow == []

    Poziom_Wydawcy.objects.create(wydawca=w1, poziom=2, rok=3333)

    denorms.flush()

    w1.refresh_from_db()
    assert w1.lista_poziomow[0] == [3333, 2]


@pytest.mark.django_db
def test_wydawca_str():
    w1 = baker.make(Wydawca, nazwa="Foo")
    w2 = baker.make(Wydawca, nazwa="Bar", alias_dla=w1)

    assert str(w2) == "Bar (alias dla Foo)"



# -*- coding: utf-8 -*-


from django.db import migrations, models
import django.core.validators
import bpp.models.profile


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0035_nomorefixtures_data_migration'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='bppuser',
            managers=[
                ('objects', bpp.models.profile.BppUserManager()),
            ],
        ),
        migrations.AlterField(
            model_name='autor',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='email',
            field=models.EmailField(max_length=254, verbose_name='email address', blank=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups'),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='last_login',
            field=models.DateTimeField(null=True, verbose_name='last login', blank=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='username',
            field=models.CharField(error_messages={'unique': 'A user with that username already exists.'}, max_length=30, validators=[django.core.validators.RegexValidator('^[\\w.@+-]+$', 'Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.', 'invalid')], help_text='Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.', unique=True, verbose_name='username'),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='patent',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='patent',
            name='utworzono',
            field=models.DateTimeField(auto_now_add=True, verbose_name=b'Utworzono', null=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='autor',
            field=models.OneToOneField(to='bpp.Autor', on_delete=models.CASCADE),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='utworzono',
            field=models.DateTimeField(auto_now_add=True, verbose_name=b'Utworzono', null=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='autor',
            field=models.OneToOneField(to='bpp.Autor', on_delete=django.db.models.deletion.CASCADE),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='utworzono',
            field=models.DateTimeField(auto_now_add=True, verbose_name=b'Utworzono', null=True),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='utworzono',
            field=models.DateTimeField(auto_now_add=True, verbose_name=b'Utworzono', null=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='utworzono',
            field=models.DateTimeField(auto_now_add=True, verbose_name=b'Utworzono', null=True),
        ),
        migrations.AlterField(
            model_name='wydzial',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='ostatnio_zmieniony',
            field=models.DateTimeField(db_index=True, auto_now=True, null=True),
        ),
    ]



# -*- encoding: utf-8 -*-

from django.http import JsonResponse
from django.views.generic.base import View
from sentry_sdk import capture_exception

from import_common.normalization import normalize_doi, normalize_isbn
from pbn_api.exceptions import NeedsPBNAuthorisationException
from pbn_api.integrator import _pobierz_prace_po_elemencie
from pbn_api.models import Publication

from bpp.models import Uczelnia
from bpp.views.api.const import API_BRAK_PARAMETRU


class GetPBNPublicationsByBase(View):
    def get_rok(
        self,
        request,
    ):
        rok = request.POST.get("rok", "").strip()[:1024]
        try:
            rok = int(rok)
        except BaseException:
            rok = None
        return rok

    def get_publication(self, value, rok):
        """Pobierz publikacje po danych z zapytania uzytkownika"""
        return Publication.objects.get(
            **{self.publication_element_name: value, "year": rok}
        ).pk

    def get_normalized_values(self, request):
        # tytul = request.POST.get("tytul")
        return self.get_value(request), self.get_rok(request)

    def get_publication_by_element_name(
        self, client, elementName, normalizedInput, year
    ):
        return _pobierz_prace_po_elemencie(
            client,
            elementName,
            normalizedInput,
            # title=tytul[: len(tytul) // 3 * 4],
            year=year,
            matchuj=True,
        )

    def post(self, request, *args, **kw):
        ni, rok = self.get_normalized_values(request)

        if not ni:
            return JsonResponse({"error": API_BRAK_PARAMETRU})

        uczelnia = Uczelnia.objects.get_default()
        if not uczelnia:
            return JsonResponse({"error": "W systemie brak obiektu Uczelnia"})

        # Zawsze pobierz świeże dane z serwera -- w pierwszej kolejności:

        client = uczelnia.pbn_client(request.user.pbn_token)

        try:
            ret = self.get_publication_by_element_name(
                client=client,
                elementName=self.publication_element_name,
                normalizedInput=ni,
                year=rok,
            )
        except NeedsPBNAuthorisationException:
            return JsonResponse(
                {"error": "Autoryzuj się w PBN korzystając z menu na głównej stronie. "}
            )
        except Exception as e:
            # Zgłoś nieznany typ błędu do Sentry
            capture_exception(e)

            # .. oraz do użytkownika:
            return JsonResponse(
                {
                    "error": "Nieznany błąd po stronie serwera przy wywołaniu funkcji odpytującej PBN. "
                    + str(e)
                }
            )

        if ret:
            return JsonResponse({"id": ret[0].pbn_uid.pk})

        # Jeżeli serwer nie zwrócil nic rozsądnego, zwróc dane z lokalnego cache
        try:
            return JsonResponse({"id": self.get_publication(ni, rok)})
        except Publication.DoesNotExist:
            return JsonResponse(
                {
                    "error": "Taka publikacja nie istnieje ani w bazie PBN, ani w lokalnym "
                    "cache rekordów PBNu. "
                }
            )
        except Publication.MultipleObjectsReturned:
            # Wiele rekordów, zwróc ISBN. Po stronie klienta spowoduje to wpisanie numeru ISBN
            # do pola wyszukujacego
            return JsonResponse({"id": ni})

        raise NotImplementedError("Should never happen")


class GetPBNPublicationsByISBN(GetPBNPublicationsByBase):
    publication_element_name = "isbn"  # nazwa parametru dla _pobierz_prace_po_elemencie

    def get_value(self, request):
        """Pobierz wartość dla zapytania (ISBN, DOI, itp)"""
        ni = normalize_isbn(request.POST.get("t", "").strip()[:1024])
        return ni


class GetPBNPublicationsByDOI(GetPBNPublicationsByBase):
    publication_element_name = "doi"  # nazwa parametru dla _pobierz_prace_po_elemencie

    def get_value(self, request):
        """Pobierz wartość dla zapytania (ISBN, DOI, itp)"""
        ni = normalize_doi(request.POST.get("t", "").strip()[:1024])
        return ni



# -*- encoding: utf-8 -*-
from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import BppMultiseekVisibility
from bpp.multiseek_registry import registry


class Command(BaseCommand):
    help = "Resetuje sortowanie opci w wyszukiwarce zgodnie z kodem"

    @transaction.atomic
    def handle(self, *args, **options):
        # Tu użyjemy registry.fields, gdyż registry.get_fields() zwraca pola w kolejności takiej,
        # jak w bazie danych. registry.fields zawiera pola tak, jak zostały zadeklarowane w kodzie.
        for no, elem in enumerate(registry.fields, 1):
            try:
                field_name = elem.bpp_multiseek_visibility_field_name
            except AttributeError:
                field_name = elem.field_name

            try:
                i = BppMultiseekVisibility.objects.get(field_name=field_name)
            except BppMultiseekVisibility.DoesNotExist:
                continue

            if i.sort_order != no:
                i.sort_order = no
                i.save()



import os
import shutil
from os.path import basename
from tempfile import mkdtemp

import denorm
from django.core.files import File
from django.core.management import call_command
from django.db.models import Sum

from ewaluacja2021.models import ZamowienieNaRaport
from ewaluacja2021.reports import load_data, rekordy
from ewaluacja2021.util import string2fn

from bpp.models import Patent_Autor, Wydawnictwo_Ciagle_Autor, Wydawnictwo_Zwarte_Autor

from django_bpp import celery_tasks


def suma_odpietych_dyscyplin():
    return (
        Wydawnictwo_Ciagle_Autor.objects.exclude(dyscyplina_naukowa=None)
        .exclude(przypieta=True)
        .count()
        + Wydawnictwo_Zwarte_Autor.objects.exclude(dyscyplina_naukowa=None)
        .exclude(przypieta=True)
        .count()
        + Patent_Autor.objects.exclude(dyscyplina_naukowa=None)
        .exclude(przypieta=True)
        .count()
    )


def suma_pkdaut_json(fn):
    dane = load_data(open(fn))
    rec = rekordy(dane)
    suma_pkdaut = rec.filter(do_ewaluacji=True).aggregate(x=Sum("pkdaut"))
    return suma_pkdaut["x"]


KLASY = Wydawnictwo_Zwarte_Autor, Wydawnictwo_Ciagle_Autor, Patent_Autor


def zrob_liste_odpietych():
    ret = {}
    for klass in KLASY:
        ret[klass] = set(
            klass.objects.exclude(dyscyplina_naukowa=None)
            .exclude(przypieta=True)
            .values_list("id", flat=True)
        )
    return ret


def przywroc_przypiecia(odpiete_przed, odpiete_po):
    for klass in KLASY:
        przed = odpiete_przed[klass]
        po = odpiete_po[klass]
        klass.objects.filter(pk__in=po.difference(przed)).update(przypieta=True)


@celery_tasks.app.task
def generuj_algorytm(pk, *args, **kw):
    zamowienie = ZamowienieNaRaport.objects.get(pk=pk)
    zamowienie.uid_zadania = generuj_algorytm.request.id
    zamowienie.status = "Przetwarzanie"
    zamowienie.save()

    odpinaj_dyscypliny = False

    kwargs = {}
    if zamowienie.rodzaj == "plecakowy":
        rodzaj_zamowienia = "plecakowy"
    elif zamowienie.rodzaj == "plecakowy_bez_limitu":
        rodzaj_zamowienia = "plecakowy"
        kwargs["bez_limitu_uczelni"] = True
    elif zamowienie.rodzaj == "genetyczny":
        rodzaj_zamowienia = "genetyczny"
    elif zamowienie.rodzaj == "genetyczny_z_odpinaniem":
        rodzaj_zamowienia = "genetyczny"
        odpinaj_dyscypliny = True
    else:
        raise NotImplementedError(
            f"Nie mam kodu dla rodzaju zamowienia {zamowienie.rodzaj}"
        )

    suma_pkdaut = poprzedni_wynik = 0
    rekordy_odpiete_przed = set()
    rekordy_odpiete_po = set()

    while True:
        outdir = mkdtemp()

        call_command(
            f"raport_3n_{rodzaj_zamowienia}",
            dyscyplina=zamowienie.dyscyplina_naukowa.nazwa,
            output_path=outdir,
            **kwargs,
        )

        json_file = os.path.join(
            outdir,
            f"{rodzaj_zamowienia}_{string2fn(zamowienie.dyscyplina_naukowa.nazwa)}.json",
        )

        suma_pkdaut = suma_pkdaut_json(json_file)

        if suma_pkdaut < poprzedni_wynik:
            # poprzendi wynik był lepszy, przywracam przypięcia, wychodze z programu
            odpinaj_dyscypliny = False
            przywroc_przypiecia(rekordy_odpiete_przed, rekordy_odpiete_po)
            denorm.flush()
            suma_pkdaut = poprzedni_wynik
        else:
            poprzedni_wynik = suma_pkdaut
            zamowienie.status = "Przetwarzanie! Osiagnieto %s PKD" % suma_pkdaut
            # zamowienie.save()

            call_command("raport_3n_to_xlsx", json_file)

            xls_output_dir = json_file.replace(".json", "_output")
            zip_path = os.path.join(outdir, "results.zip")

            cwd = os.getcwd()
            os.chdir(outdir)
            os.system(f'zip -r "{zip_path}" "{os.path.basename(xls_output_dir)}/"')
            os.chdir(cwd)

            zamowienie.plik_wyjsciowy.save(
                basename(xls_output_dir) + ".zip", content=File(open(zip_path, "rb"))
            )

            fitness_png = os.path.join(outdir, "fitness.png")

            if os.path.exists(fitness_png):
                zamowienie.wykres_wyjsciowy.save(
                    basename(xls_output_dir), content=File(open(fitness_png, "rb"))
                )

            zamowienie.save()

        jeszcze_jedna_petla = False

        if odpinaj_dyscypliny:
            rekordy_odpiete_przed = zrob_liste_odpietych()
            odpiete_przed = suma_odpietych_dyscyplin()

            call_command("odepnij_dyscypliny", json_file)

            rekordy_odpiete_po = zrob_liste_odpietych()
            odpiete_po = suma_odpietych_dyscyplin()

            if odpiete_przed != odpiete_po:
                zamowienie.status = (
                    f"Przetwarzanie! Odpieto kolejne {odpiete_po-odpiete_przed} dyscyplin. Osiagnieto %s PKD"
                    % suma_pkdaut
                )
                zamowienie.save()

                jeszcze_jedna_petla = True

        shutil.rmtree(outdir)

        if not jeszcze_jedna_petla:
            break

    zamowienie.status = f"Zakonczono! Osiagnieto {suma_pkdaut}"
    zamowienie.save()



# Generated by Django 3.0.14 on 2021-08-05 21:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0018_auto_20210714_0104"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="doi",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publication",
            name="isbn",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publication",
            name="publicUri",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publication",
            name="title",
            field=models.TextField(blank=True, db_index=True, null=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="institution",
            name="addressCity",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="institution",
            name="addressPostalCode",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="institution",
            name="addressStreet",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="institution",
            name="addressStreetNumber",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="institution",
            name="name",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="institution",
            name="polonUid",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="journal",
            name="eissn",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="journal",
            name="issn",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="journal",
            name="mniswId",
            field=models.IntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="journal",
            name="title",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="journal",
            name="websiteLink",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publication",
            name="year",
            field=models.IntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publisher",
            name="mniswId",
            field=models.IntegerField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="publisher",
            name="publisherName",
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="lastName",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="name",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="orcid",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="pbnId",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="polonUid",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="scientist",
            name="qualifications",
            field=models.TextField(blank=True, null=True, verbose_name="Tytuł"),
        ),
    ]



import pytest

from bpp.models import Cache_Punktacja_Autora_Query
from raport_slotow.util import create_temporary_table_as, drop_table, clone_temporary_table


@pytest.mark.django_db
def test_util_create_tepmporary_table_as():
    c = Cache_Punktacja_Autora_Query.objects.all()
    create_temporary_table_as("foobar", c)
    create_temporary_table_as("foobar", c)  # sprawdź czy kasuje przed stworzeniem
    from django.db import DEFAULT_DB_ALIAS, connections
    connection = connections[DEFAULT_DB_ALIAS]
    with connection.cursor() as cursor:
        cursor.execute("SELECT * FROM foobar")


@pytest.mark.django_db
def test_util_drop_table():
    c = Cache_Punktacja_Autora_Query.objects.all()
    create_temporary_table_as("foobar", c)
    drop_table("foobar")


@pytest.mark.django_db
def test_clone_tmporary_table():
    c = Cache_Punktacja_Autora_Query.objects.all()
    create_temporary_table_as("foobar", c)
    clone_temporary_table("foobar", "foobar2")
    clone_temporary_table("foobar", "foobar2")
    drop_table("foobar")
    drop_table("foobar2")



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0038_autor_pesel_md5'),
        ('bpp', '0039_wydzial_archiwalny'),
    ]

    operations = [
    ]



import pytest
from model_bakery import baker

from raport_slotow.models.uczelnia import RaportSlotowUczelnia

from bpp.models import Cache_Punktacja_Autora_Query, Cache_Punktacja_Dyscypliny, Rekord


@pytest.mark.django_db
@pytest.fixture
def rekord_slotu(
    autor_jan_kowalski, jednostka, dyscyplina1, wydawnictwo_ciagle_z_autorem, rok
):
    wydawnictwo_ciagle_z_autorem.autorzy_set.update(dyscyplina_naukowa=dyscyplina1)

    rekord = Rekord.objects.get_for_model(wydawnictwo_ciagle_z_autorem)
    Cache_Punktacja_Dyscypliny.objects.create(
        rekord_id=rekord.pk,
        dyscyplina=dyscyplina1,
        pkd=50,
        slot=20,
        autorzy_z_dyscypliny=[
            autor_jan_kowalski.pk,
        ],
        zapisani_autorzy_z_dyscypliny=[
            "Foo",
        ],
    )
    return Cache_Punktacja_Autora_Query.objects.create(
        autor=autor_jan_kowalski,
        jednostka=jednostka,
        dyscyplina=dyscyplina1,
        pkdaut=50,
        slot=20,
        rekord=rekord,
    )


@pytest.mark.django_db
@pytest.fixture
def raport_slotow_uczelnia(db):
    return baker.make(RaportSlotowUczelnia)



import json


class PraceSerwisoweException(Exception):
    pass


class HttpException(Exception):
    def __init__(self, status_code, url, content):
        self.status_code = status_code
        self.url = url
        self.content = content
        try:
            self.json = json.loads(content[:4096])
        except (json.JSONDecodeError, ValueError, TypeError):
            self.json = None


class AccessDeniedException(Exception):
    def __init__(self, url, content):
        self.url = url
        self.content = content


class SciencistDoesNotExist(Exception):
    pass


class AuthenticationResponseError(Exception):
    pass


class IntegracjaWylaczonaException(Exception):
    pass


class SameDataUploadedRecently(Exception):
    pass


class WillNotExportError(Exception):
    pass


class DOIorWWWMissing(WillNotExportError):
    pass


class LanguageMissingPBNUID(WillNotExportError):
    pass


class StatementsMissing(WillNotExportError):
    pass


class PKZeroExportDisabled(WillNotExportError):
    pass


class CharakterFormalnyMissingPBNUID(WillNotExportError):
    pass


class StatementDeletionError(Exception):
    def __init__(self, status_code, url, content):
        self.status_code = status_code
        self.url = url
        self.content = content


class NeedsPBNAuthorisationException(HttpException):
    pass



import os
import random
import string
import sys
from datetime import timedelta

import environ
import sentry_sdk
from django.core.exceptions import DisallowedHost, ImproperlyConfigured
from sentry_sdk.integrations.django import DjangoIntegration

from bpp.util import slugify_function

from django_bpp.version import VERSION

# Ponieważ konieczna jest konfiguracja django-ldap-auth i potrzebne będą kolejne zmienne
# środowiskowe, ponieważ z Pipenv przeszedłem na poetry, ponieważ tych konfiguracji i
# serwerów (testowych, produkcyjnych) robi się coraz więcej -- z tych okazji zaczynam
# migrację konfiguracji na django-environ. Na ten moment przez django-environ pobiorę
# konfigurację LDAPa, docelowo przez django-environ powinna iść cała konfiguracja
# serwisu + docelowo będzie pewnie można zrezygnować z wielu plików konfiguracyjnych
# (local, test, production).

env = environ.Env(
    # casting, default value
    #
    # LDAP
    #
    AUTH_LDAP_SERVER_URI=(str, None),
    # AUTH_LDAP_BIND_DN=(str, None),
    # AUTH_LDAP_BIND_PASSWORD=(str, None),
    # AUTH_LDAP_USER_SEARCH=(str, None),
    AUTH_LDAP_GROUP_SEARCH=(str, "ou=django,ou=groups,dc=auth,dc=local"),
    AUTH_LDAP_USER_SEARCH_QUERY=(str, "userPrincipalName=%(user)s@auth.local"),
    #
    # Email
    #
    EMAIL_URL=(str, "smtp://127.0.0.1:25"),
    DEFAULT_FROM_EMAIL=(str, "webmaster@localhost"),
    SERVER_EMAIL=(str, "root@localhost"),
    #
    # Administratorzy
    #
    ADMINS=(str, "Michał Pasternak <michal.dtz@gmail.com>"),
    #
    # Konfiguracja Sentry
    #
    SENTRYSDK_CONFIG_URL=(str, None),
    SENTRYSDK_TRACES_SAMPLE_RATE=(float, 0.2),
)

ENVFILE_PATH = os.path.join(os.path.expanduser("~"), ".env")

if os.path.exists(ENVFILE_PATH) and os.path.isfile(ENVFILE_PATH):
    environ.Env.read_env(ENVFILE_PATH)

#
# "Stara" funkcja django_getenv, pobierająca zmienne srodowiskowe
#


def django_getenv(varname, default=None, type=str):
    value = os.getenv(varname, default)

    if value is None:
        raise ImproperlyConfigured(f"Please set {varname} variable")

    try:
        value = type(value)
    except (ValueError, TypeError):
        raise ImproperlyConfigured(f"Cannot convert variable {varname} to type {type}")

    return value


#
# Ustaw Sentry
#


SENTRYSDK_CONFIG_URL = (
    env("SENTRYSDK_CONFIG_URL")
    or os.getenv("DJANGO_BPP_RAVEN_CONFIG_URL", None)
    or os.getenv("DJANGO_BPP_SENTRYSDK_CONFIG_URL", None)
)

PROCESS_INTERACTIVE = sys.stdin.isatty()

if SENTRYSDK_CONFIG_URL and not PROCESS_INTERACTIVE:
    sentry_sdk.init(
        dsn=SENTRYSDK_CONFIG_URL,
        traces_sample_rate=env("SENTRYSDK_TRACES_SAMPLE_RATE"),
        integrations=[DjangoIntegration()],
        release=VERSION,
        ignore_errors=[DisallowedHost],
        send_default_pii=True,
    )


BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))


TIME_ZONE = "Europe/Warsaw"
LANGUAGE_CODE = "pl"
LANGUAGES = (("pl", "Polish"),)

SITE_ID = 1  # dla static-sitemaps
USE_I18N = True
USE_L10N = True
USE_TZ = True

STATIC_URL = "/static/"

ADMIN_MEDIA_PREFIX = "/static/admin/"
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "bpp.finders.YarnFinder",
    "compressor.finders.CompressorFinder",
    #    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = "=6uqi1)(qnzjo8q@-@m#egd8v#+zac6feh2h-b&amp;=3bczpfqxxd"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(BASE_DIR, "templates"),
        ],
        "OPTIONS": {
            "loaders": [
                "admin_tools.template_loaders.Loader",
                "dbtemplates.loader.Loader",
                # Taka kolejnosc "laoders", i nie cache'ujemy tych dwóch powyżej, bo:
                # - dbtemplates ma swoje cacheowanie i uzywa wbudowanego backendu
                # - wbudowany backend domyslnie to 'locmem'
                # - jezeli uruchomie denorm_queue i serwer, to nie ma wymiany danych miedzy
                #   nimi obydwoma.
                # - pliki moga sie cache'owac, zarowno na testach jak i na produkcji,
                # - ... gdyz na testach jest DummyCacheBackend, a produkcja uzywa redis.
                # - rozchodzi sie o to, ze denorm_queue i reszta procesow musi miec podobna
                #   zawartosc cache-backendu,
                # - ... ktorej nie zapewni LocMem, bo on jest tylko na jeden proces.
                (
                    "django.template.loaders.cached.Loader",
                    [
                        "django.template.loaders.filesystem.Loader",
                        "django.template.loaders.app_directories.Loader",
                    ],
                ),
            ],
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.request",
                "django.template.context_processors.static",
                "django.contrib.messages.context_processors.messages",
                "password_policies.context_processors.password_status",
                "bpp.context_processors.uczelnia.uczelnia",
                "bpp.context_processors.config.theme_name",
                "bpp.context_processors.config.enable_new_reports",
                "bpp.context_processors.config.max_no_authors_on_browse_jednostka_page",
                "bpp.context_processors.global_nav.user",
                "bpp.context_processors.google_analytics.google_analytics",
            ],
        },
    },
]

MIDDLEWARE = [
    "htmlmin.middleware.HtmlMinifyMiddleware",
    "htmlmin.middleware.MarkRequestMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    # "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "password_policies.middleware.PasswordChangeMiddleware",
    "dj_pagination.middleware.PaginationMiddleware",
    "session_security.middleware.SessionSecurityMiddleware",
    "notifications.middleware.NotificationsMiddleware",
]

INTERNAL_IPS = ("127.0.0.1",)

ROOT_URLCONF = "django_bpp.urls"

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = "django_bpp.wsgi.application"


def _elem_in_sys_argv(possible):
    argv = " ".join(sys.argv)
    for elem in possible:
        if elem in argv:
            return True


TESTING = _elem_in_sys_argv(
    [
        "jenkins",
        "py.test",
        "pytest",
        "helpers/pycharm/_jb_pytest_runner.py",
        "manage.py test",
    ]
)
MIGRATING = _elem_in_sys_argv(["makemigrations", "migrate"])

if TESTING:
    CELERY_ALWAYS_EAGER = True
    CELERY_TASK_ALWAYS_EAGER = True
    CELERY_EAGER_PROPAGATES_EXCEPTIONS = True


INSTALLED_APPS = [
    "tee",
    "formtools",
    "denorm.apps.DenormAppConfig",
    "reversion",
    "djangoql",
    "cacheops",
    "channels",
    "dynamic_columns",
    "django.contrib.humanize",
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.sitemaps",
    "django.contrib.postgres",
    "long_running",
    "import_pracownikow",
    "import_list_if",
    "password_policies",
    "create_test_db",
    "celery",
    "django_celery_results",
    "flexible_reports",
    "static_sitemaps",
    "cookielaw",
    "taggit",
    "taggit_serializer",
    "columns",
    "zglos_publikacje.apps.ZglosPublikacjeConfig",
    "formdefaults.apps.FormdefaultsConfig",
    "raport_slotow",
    # Musi być PRZED django-autocomplete-light do momentu
    # dal 3.3.0-release, musi być naprawiony o ten błąd:
    # https://github.com/yourlabs/django-autocomplete-light/issues/981
    "bpp",
    "crossref_bpp",
    "pbn_api",
    "dal",
    "dal_select2",
    "grappelli",
    "django_bpp.apps.BppAdminConfig",  # replaced `django.contrib.admin`
    "permissions_widget",
    "dj_pagination",
    "admin_tools",
    "admin_tools.theming",
    "admin_tools.menu",
    "admin_tools.dashboard",
    "django_tables2",
    # 'FOR_REMOVAL_autocomplete_light',
    "messages_extends",
    "multiseek",
    "django_extensions",
    "celeryui",
    "crispy_forms",
    "crispy_forms_foundation",
    "compressor",
    "session_security",
    "notifications",
    "integrator2",
    "nowe_raporty",
    "rozbieznosci_dyscyplin",
    "loginas",
    "rozbieznosci_if",
    "robots",
    "webmaster_verification",
    "favicon",
    "miniblog",
    "import_dyscyplin",
    "mptt",
    "import_dbf",
    "rest_framework",
    "django_filters",
    "api_v1",
    "adminsortable2",
    "import_export",
    "ewaluacja2021",
    # Zostawiamy - bezwarunkowo
    "test_bpp",
    #
    "dbtemplates",
]

# Profile użytkowników
AUTH_USER_MODEL = "bpp.BppUser"

GRAPPELLI_INDEX_DASHBOARD = "django_bpp.dashboard.CustomIndexDashboard"
GRAPPELLI_ADMIN_TITLE = "BPP"
ADMIN_TOOLS_MENU = "django_bpp.menu.CustomMenu"

PROJECT_ROOT = BASE_DIR

MULTISEEK_REGISTRY = "bpp.multiseek_registry"


AUTOSLUG_SLUGIFY_FUNCTION = slugify_function

LOGIN_REDIRECT_URL = "/"

LOGOUT_REDIRECT_URL = LOGIN_REDIRECT_URL

# django
MEDIA_URL = "/media/"

INTERNAL_IPS = ("127.0.0.1",)

# djorm-pool
DJORM_POOL_OPTIONS = {
    "pool_size": 30,
    "max_overflow": 0,
    "recycle": 3600,  # the default value
}


TEST_RUNNER = "django.test.runner.DiscoverRunner"

PROJECT_APPS = ("bpp",)


# Ustawienia ModelBakery


def autoslug_gen():
    return "".join(
        random.choice(string.ascii_uppercase + string.digits) for _ in range(50)
    )


BAKER_CUSTOM_FIELDS_GEN = {"autoslug.fields.AutoSlugField": autoslug_gen}
BAKER_CUSTOM_CLASS = "bpp.tests.bpp_baker.BPP_Baker"

SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTOCOL", "https")

MAT_VIEW_REFRESH_COUNTDOWN = 30

SCRIPT_PATH = os.path.abspath(os.path.dirname(__file__))

SITE_ROOT = os.path.abspath(os.path.join(SCRIPT_PATH, "..", ".."))

STATIC_ROOT = os.path.abspath(os.path.join(SCRIPT_PATH, "..", "staticroot"))

COMPRESS_CSS_FILTERS = [
    "compressor.filters.css_default.CssAbsoluteFilter",
    "compressor.filters.cssmin.CSSMinFilter",
]
COMPRESS_JS_FILTERS = ["compressor.filters.jsmin.rJSMinFilter"]

COMPRESS_ROOT = STATIC_ROOT
COMPRESS_OFFLINE_CONTEXT = [
    {
        "THEME_NAME": "scss/app-blue.css",
        "STATIC_URL": STATIC_URL,
        "LANGUAGE_CODE": "pl",
    },
    {
        "THEME_NAME": "scss/app-green.css",
        "STATIC_URL": STATIC_URL,
        "LANGUAGE_CODE": "pl",
    },
    {
        "THEME_NAME": "scss/app-orange.css",
        "STATIC_URL": STATIC_URL,
        "LANGUAGE_CODE": "pl",
    },
]

# Domyslnie, redis na Ubuntu pozwala na 16 baz danych
REDIS_DB_BROKER = django_getenv("DJANGO_BPP_REDIS_DB_BROKER", 1, int)
REDIS_DB_CELERY = django_getenv("DJANGO_BPP_REDIS_DB_CELERY", 2, int)
REDIS_DB_SESSION = django_getenv("DJANGO_BPP_REDIS_DB_SESSION", 4, int)
REDIS_DB_CACHE = django_getenv("DJANGO_BPP_REDIS_DB_CACHE", 5, int)
REDIS_DB_LOCKS = django_getenv("DJANGO_BPP_REDIS_DB_LOCKS", 6, int)

ALLOWED_HOSTS = [
    "127.0.0.1",
    "test.unexistenttld",
    django_getenv("DJANGO_BPP_HOSTNAME", "localhost"),
]

REDIS_HOST = django_getenv("DJANGO_BPP_REDIS_HOST", "localhost")
REDIS_PORT = int(django_getenv("DJANGO_BPP_REDIS_PORT", "6379"))

BROKER_URL = f"redis://{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB_BROKER}"
# CELERY_RESULT_BACKEND = BROKER_URL
CELERY_RESULT_BACKEND = "django-db"

#
SESSION_REDIS_HOST = REDIS_HOST
SESSION_REDIS_PORT = REDIS_PORT
SESSION_REDIS_DB = REDIS_DB_SESSION
SESSION_REDIS_PREFIX = "session"

ALLOWED_TAGS = ("b", "em", "i", "strong", "strike", "u", "sup", "font", "sub")

SESSION_SECURITY_PASSIVE_URLS = ["/messages/"]


def int_or_None(value):
    try:
        return int(value)
    except ValueError:
        return ""


DATABASES = {
    "default": {
        "ENGINE": django_getenv(
            "DJANGO_BPP_DB_ENGINE", "django.db.backends.postgresql_psycopg2"
        ),
        "NAME": django_getenv("DJANGO_BPP_DB_NAME", "bpp"),
        "USER": django_getenv("DJANGO_BPP_DB_USER", "postgres"),
        "PASSWORD": django_getenv("DJANGO_BPP_DB_PASSWORD", "password"),
        "HOST": django_getenv("DJANGO_BPP_DB_HOST", "localhost"),
        "PORT": int_or_None(django_getenv("DJANGO_BPP_DB_PORT", "5432")),
    },
}

if DATABASES["default"]["HOST"] in ["localhost", "127.0.0.1"]:
    options = DATABASES["default"].get("OPTIONS", {})
    options["sslmode"] = "disable"
    DATABASES["default"]["OPTIONS"] = options

SECRET_KEY = django_getenv("DJANGO_BPP_SECRET_KEY")

SENDFILE_URL = MEDIA_URL

# django-password-policies

# Zmiana hasla co 30 dni
PASSWORD_DURATION_SECONDS = int(
    os.getenv("DJANGO_BPP_PASSWORD_DURATION_SECONDS", str((60 * 60 * 24) * 30))
)

PASSWORD_USE_HISTORY = bool(int(os.getenv("DJANGO_BPP_USE_PASSWORD_HISTORY", "1")))

PASSWORD_HISTORY_COUNT = int(os.getenv("DJANGO_BPP_PASSWORD_HISTORY_COUNT", "12"))

# wymagane przez django-password-policies
SESSION_SERIALIZER = "django.contrib.sessions.serializers.PickleSerializer"

MESSAGE_STORAGE = "messages_extends.storages.FallbackStorage"

TEST_NON_SERIALIZED_APPS = ["django.contrib.contenttypes", "django.contrib.auth"]

CELERYD_HIJACK_ROOT_LOGGER = False

CELERY_TRACK_STARTED = True

CELERYBEAT_SCHEDULE = {
    "cleanup-integrator2-files": {
        "task": "integrator2.tasks.remove_old_integrator_files",
        "schedule": timedelta(days=1),
    },
    "cleanup-report-files": {
        "task": "bpp.tasks.remove_old_report_files",
        "schedule": timedelta(days=1),
    },
    "zaktualizuj-liczbe-cytowan": {
        "task": "bpp.tasks.zaktualizuj_liczbe_cytowan",
        "schedule": timedelta(days=5),
    },
}


def can_login_as(request, target_user):
    return request.user.is_superuser


CAN_LOGIN_AS = can_login_as

#

MEDIA_ROOT = django_getenv("DJANGO_BPP_MEDIA_ROOT", os.getenv("HOME", "C:/bpp-media"))

SENDFILE_ROOT = MEDIA_ROOT

GOOGLE_ANALYTICS_PROPERTY_ID = django_getenv(
    "DJANGO_BPP_GOOGLE_ANALYTICS_PROPERTY_ID", ""
)

ROBOTS_SITEMAP_VIEW_NAME = "sitemap"

WEBMASTER_VERIFICATION = {
    "google": django_getenv("DJANGO_BPP_GOOGLE_VERIFICATION_CODE", "1111111111111111"),
}

EXCLUDE_FROM_MINIFYING = ["^google.*html$"]

SESSION_EXPIRE_AT_BROWSER_CLOSE = True

UZYWAJ_PUNKTACJI_WEWNETRZNEJ = bool(
    int(django_getenv("DJANGO_BPP_UZYWAJ_PUNKTACJI_WEWNETRZNEJ", "1"))
)

THEME_NAME = os.getenv("DJANGO_BPP_THEME_NAME", "app-blue")

ENABLE_NEW_REPORTS = int(os.getenv("DJANGO_BPP_ENABLE_NEW_REPORTS", "1"))

SESSION_SECURITY_WARN_AFTER = int(
    os.getenv("DJANGO_BPP_SESSION_SECURITY_WARN_AFTER", "540")
)

SESSION_SECURITY_EXPIRE_AFTER = int(
    os.getenv("DJANGO_BPP_SESSION_SECURITY_EXPIRE_AFTER", "600")
)

PUNKTUJ_MONOGRAFIE = bool(int(os.getenv("DJANGO_BPP_PUNKTUJ_MONOGRAFIE", "1")))

STATICSITEMAPS_ROOT_SITEMAP = "django_bpp.sitemaps.django_bpp_sitemaps"
STATICSITEMAPS_REFRESH_AFTER = 24 * 60

# dla django-model-utils SplitField
SPLIT_MARKER = "<!-- tutaj -->"

# django-crispy-forms: użyj crispy-forms-foundation

CRISPY_ALLOWED_TEMPLATE_PACKS = "foundation-6"

CRISPY_TEMPLATE_PACK = "foundation-6"

CRISPY_CLASS_CONVERTERS = {
    "inputelement": None,
    "errorcondition": "is-invalid-input",
}

SILENCED_SYSTEM_CHECKS = ["urls.W003"]

# Sposób generowania inline dla powiązań rekordu autora z rekordem
# publikacji

INLINE_DLA_AUTOROW = os.getenv("DJANGO_BPP_INLINE_DLA_AUTOROW", "stacked")

DEBUG_TOOLBAR = False

BPP_DODAWAJ_JEDNOSTKE_PRZY_ZAPISIE_PRACY = True

YARN_FILE_PATTERNS = {
    "jquery": ["dist/jquery.min.js"],
    "jqueryui": ["jquery-ui.min.js", "jquery-ui.css"],
    "what-input": ["dist/what-input.js"],
    "jquery.cookie": ["jquery.cookie.js"],
    "foundation-sites": [
        "dist/js/foundation.min.js",
        "dist/js/foundation.min.js.map",
        # testy
        "dist/css/normalize.min.css",
        "dist/css/foundation.min.css",
    ],
    "foundation-datepicker": [
        "foundation/fonts/*",
        "css/foundation-datepicker.min.css",
        "js/foundation-datepicker.min.js",
        "js/locales/foundation-datepicker.pl.js",
    ],
    "datatables.net": ["js/jquery.dataTables.js"],
    "datatables.net-zf": [
        "js/dataTables.foundation.js",
        "css/dataTables.foundation.css",
    ],
    "select2": [
        "dist/css/select2.min.css",
        "dist/js/select2.full.min.js",
        "dist/js/i18n/pl.js",
    ],
    "jinplace": ["js/jinplace.js"],
    "select2-foundation_theme": ["dist/select2-foundation-theme.css"],
    "kbw-keypad": ["dist/*"],
    # Zostana pozniej usuniete przez MANIFEST.in
    "qunit": ["qunit/qunit.js", "qunit/qunit.css"],
    "sinon": ["pkg/sinon.js"],
}


REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly"
    ],
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.LimitOffsetPagination",
    "PAGE_SIZE": 10,
    "DEFAULT_FILTER_BACKENDS": ["django_filters.rest_framework.DjangoFilterBackend"],
    # Nie limituj ilości zapytań (mpasternak, 6.06.2020) - jednakże, gdyby
    # trzeba było, to wystarczy odkomentować poniższe dwie linie:
    # "DEFAULT_THROTTLE_CLASSES": ("rest_framework.throttling.AnonRateThrottle",),
    # "DEFAULT_THROTTLE_RATES": {"anon": "50/second",},
}


BPP_WALIDUJ_AFILIACJE_AUTOROW = (
    os.getenv("DJANGO_BPP_WALIDUJ_AFILIACJE_AUTOROW", "tak") == "tak"
)


ASGI_APPLICATION = "django_bpp.routing.application"
CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [(REDIS_HOST, REDIS_PORT)],
        },
    },
}


# "Wszytko na stdout" konfiguracja logowania poniżej

LOG_EVERYTHING = False

if LOG_EVERYTHING:
    import logging.config

    LOGGING_CONFIG = None
    logging.config.dictConfig(
        {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "console": {
                    # exact format is not important, this is the minimum information
                    "format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
                },
            },
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                    "formatter": "console",
                },
                # Add Handler for Sentry for `warning` and above
            },
            "loggers": {
                # root logger
                "": {
                    "level": "DEBUG",
                    "handlers": [
                        "console",
                    ],
                },
            },
        }
    )

# django-compressor dla każdej wersji będzie miał swoją nazwę katalogu
# wyjściowego, z tej prostej przyczyny, że nie wszystkie przeglądarki
# pamiętają, żeby odświeżyć cache:
COMPRESS_OUTPUT_DIR = f"CACHE-{VERSION}"


# django-permissions-widget
PERMISSIONS_WIDGET_PATCH_GROUPADMIN = False
PERMISSIONS_WIDGET_PATCH_USERADMIN = True
PERMISSIONS_WIDGET_EXCLUDE_APPS = [
    "import_dbf",
    "favicon",
    "taggit",
    "test_bpp",
    "rozbieznosci_if",
    "rozbieznosci_dyscyplin",
    "sessions",
    "sites",
    "contenttypes",
    "raport_slotow",
    "multiseek",
    "messages_extends",
    "menu",
    "sites",
    "robots",
    "pbn_api",
    "admin",
    "auth",
    "password_policies",
    "notifications",
    "celeryui",
    "dashboard",
    "django.contrib.contenttypes",
    "egeria",
    "eksport_pbn",
    "import_dyscyplin",
    "import_list_if",
    "import_pracownikow",
    "integrator2",
    "admin_tools",
]

PERMISSIONS_WIDGET_EXCLUDE_MODELS = [
    "bpp.cache_punktacja_autora",
    "bpp.cache_punktacja_autora_query",
    "bpp.cache_punktacja_autora_query_view",
    "bpp.cache_punktacja_autora_sum",
    "bpp.cache_punktacja_autora_sum_group_ponizej",
    "bpp.cache_punktacja_autora_sum_gruop",
    "bpp.cache_punktacja_autora_sum_ponizej",
    "bpp.cache_punktacja_dyscypliny",
    "bpp.sumy_wydawnictwo_zwarte_view",
    "bpp.opi_2012_afiliacja_do_wydzialu",
    "bpp.opi_2012_tytul_cache",
    "bpp.nowe_sumy_view",
    "bpp.autorzy",
    "bpp.rekord",
    "bpp.rekord_view",
    "bpp.autorzy_view",
]


DBTEMPLATES_USE_REVERSION = True

DENORM_DISABLE_AUTOTIME_DURING_FLUSH = True
DENORM_AUTOTIME_FIELD_NAMES = [
    "ostatnio_zmieniony",
]

MAX_NO_AUTHORS_ON_BROWSE_JEDNOSTKA_PAGE = 200
"""
Maksymalna ilość autorów wyświetlanych w danej grupie na podstronie przeglądania danych jednostki. W przypadku
przekroczenia tej liczby, dana podgrupa autorów ("aktualni pracownicy","współpracowali kiedyś" itp) nie zostanie
wyświetlona.
"""


DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# django-import-export, używaj transakcji:
IMPORT_EXPORT_USE_TRANSACTIONS = True

# Maksymalna
BPP_MAX_ALLOWED_EXPORT_ITEMS = 1500

#
# Konfiguracja LDAP
#

AUTH_LDAP_SERVER_URI = env("AUTH_LDAP_SERVER_URI")

if AUTH_LDAP_SERVER_URI:
    AUTH_LDAP_BIND_DN = env("AUTH_LDAP_BIND_DN")
    AUTH_LDAP_BIND_PASSWORD = env("AUTH_LDAP_BIND_PASSWORD")

    import ldap
    from django_auth_ldap.config import GroupOfNamesType, LDAPSearch

    AUTH_LDAP_USER_SEARCH = LDAPSearch(
        env("AUTH_LDAP_USER_SEARCH"),
        ldap.SCOPE_SUBTREE,
        env("AUTH_LDAP_USER_SEARCH_QUERY"),
    )

    # Set up the basic group parameters.
    AUTH_LDAP_GROUP_SEARCH = LDAPSearch(
        env("AUTH_LDAP_GROUP_SEARCH"),
        ldap.SCOPE_SUBTREE,
        "(objectClass=groupOfNames)",
    )
    AUTH_LDAP_GROUP_TYPE = GroupOfNamesType(name_attr="cn")

    AUTH_LDAP_GROUP_TYPE = GroupOfNamesType(name_attr="cn")

    # This is the default, but I like to be explicit.
    AUTH_LDAP_ALWAYS_UPDATE_USER = True

    # Use LDAP group membership to calculate group permissions.
    AUTH_LDAP_FIND_GROUP_PERMS = True

    # Cache distinguished names and group memberships for an hour to minimize
    # LDAP traffic.
    AUTH_LDAP_CACHE_TIMEOUT = 3600

    # Keep ModelBackend around for per-user permissions and maybe a local
    # superuser.
    AUTHENTICATION_BACKENDS = (
        "django_auth_ldap.backend.LDAPBackend",
        "django.contrib.auth.backends.ModelBackend",
    )

#
# Koniec konfiguracji LDAP
#


#
# Konfiguracja serwera pocztowego
#
EMAIL_CONFIG = env.email("EMAIL_URL")
vars().update(EMAIL_CONFIG)
SERVER_EMAIL = env("SERVER_EMAIL")
DEFAULT_FROM_EMAIL = env("DEFAULT_FROM_EMAIL")

#
# Koniec konfiguracji serwera pocztowego
#

#
# Konta administratorów i managerów
#

from email.utils import getaddresses

ADMINS = getaddresses([env("ADMINS")])
MANAGERS = ADMINS

#
# Koniec konfiguracji kont administratora i managera
#

#
# django-easy-audit
#

DJANGO_EASY_AUDIT_WATCH_REQUEST_EVENTS = False
DJANGO_EASY_AUDIT_ADMIN_SHOW_REQUEST_EVENTS = False
DJANGO_EASY_AUDIT_READONLY_EVENTS = True
DJANGO_EASY_AUDIT_REGISTERED_CLASSES = [
    "zglos_publikacje.Zgloszenie_Publikacji",
    "bpp.Wydawnictwo_Zwarte",
    "bpp.Wydawnictwo_Ciagle",
    "bpp.Patent",
    "bpp.Praca_Doktorska",
    "bpp.Praca_Habilitacyjna",
    "bpp.Autor",
    "bpp.Jednostka",
    "bpp.Uczelnia",
    "bpp.Wydzial",
]

#
# Koniec django-easy-audit
#


SILENCED_SYSTEM_CHECKS.append("admin.E117")

DYNAMIC_COLUMNS_ALLOWED_IMPORT_PATHS = [
    "bpp.admin.wydawnictwo_ciagle",
    "bpp.admin.wydawnictwo_zwarte",
    "bpp.admin.autor",
]

DYNAMIC_COLUMNS_FORBIDDEN_COLUMN_NAMES = [
    "^kc_.*",
    ".*_cache$",
    ".*_sort$",
    "search_index",
    "legacy_data",
    "slug",
    "^cached_.*",
]



# -*- coding: utf-8 -*-


from datetime import date

from django.db import migrations
from django.db.models.functions import Coalesce


def utworz_jednostka_wydzial(apps, schema_editor):
    Jednostka = apps.get_model("bpp", "Jednostka")
    Jednostka_Wydzial = apps.get_model("bpp", "Jednostka_Wydzial")
    db_alias = schema_editor.connection.alias

    for j in Jednostka.objects.using(db_alias).all():
        if Jednostka_Wydzial.objects.using(db_alias).filter(
                jednostka_id=j.pk,
                wydzial_id=j.wydzial_id).count() == 0:
            Jednostka_Wydzial.objects.using(db_alias).create(
                jednostka_id=j.pk,
                wydzial_id=j.wydzial_id,
            )


def usun_jednostka_wydzial(apps, schema_editor):
    Jednostka = apps.get_model("bpp", "Jednostka")

    Jednostka_Wydzial = apps.get_model("bpp", "Jednostka_Wydzial")
    db_alias = schema_editor.connection.alias

    for j in Jednostka.objects.using(db_alias).all():
        q = Jednostka_Wydzial.objects.using(db_alias).filter(
            jednostka_jd=j.pk,
            wydzial_id=j.wydzial_id)
        jw = q.annotate(
            od_not_null=Coalesce("od", date(1, 1, 1)),
        ).order_by("-od_not_null").first()
        q.delete()
        j.wydzial_id = jw.wydzial_id
        j.save()


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0060_auto_20161114_1905'),
    ]

    operations = [
        migrations.RunPython(utworz_jednostka_wydzial, usun_jednostka_wydzial)
    ]



import json
from collections import defaultdict
from pathlib import Path


def load_custom_sql(mig_name, app_name="bpp", charset="utf-8", *args, **kw):
    import os

    from django.db import connection

    fn = os.path.join(
        os.path.dirname(__file__), "..", app_name, "migrations", mig_name + ".sql"
    )
    # print "Loading %s... " % fn,
    data = open(fn, "rb").read().decode(charset)

    cursor = connection.cursor()
    cursor = cursor.cursor
    cursor.execute(data)
    # print "done!"


def load_fixture_as_json(fixture_name):
    return json.loads(open(str(find_fixture_path(fixture_name))).read())


def find_fixture_path(fixture):
    return Path(__file__).parent / "fixtures" / (fixture + ".json")


def load_historic_fixture(apps, fixture_name, klass, app_name="bpp"):
    max_id_map = defaultdict(int)
    for elem in load_fixture_as_json(fixture_name):
        app_name, klass = elem["model"].split(".")
        klassobj = apps.get_model(app_name, klass)
        kw = elem["fields"]
        pk = int(elem["pk"])
        kw["pk"] = pk
        max_id_map[klassobj] = max(max_id_map[klassobj], pk)
        klassobj.objects.create(**kw)

    from django.db import connection

    cur = connection.cursor()
    for klassobj, cnt in max_id_map.items():
        qry = f"ALTER SEQUENCE { klassobj._meta.db_table }_id_seq RESTART WITH { cnt + 1 }"
        cur.execute(qry)



# -*- encoding: utf-8 -*-

"""Funkcje pomocnicze dla klas w bpp.models"""
from django.core.exceptions import ObjectDoesNotExist, ValidationError

from bpp.models.szablondlaopisubibliograficznego import SzablonDlaOpisuBibliograficznego

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.db import models
from django.db.models import Max
from django.template.loader import get_template

from django.utils import safestring


def dodaj_autora(
    klass,
    rekord,
    autor,
    jednostka,
    zapisany_jako=None,
    typ_odpowiedzialnosci_skrot="aut.",
    kolejnosc=None,
    dyscyplina_naukowa=None,
    afiliuje=True,
):
    """
    Utility function, dodająca autora do danego rodzaju klasy (Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte, Patent); funkcja używana przez te klasy, niejako
    wewnętrzna dla całego API; nie powinna być używana bezpośrednio nigdzie,
    jedynie API tych klas winno być używane.

    :param klass:
    :param rekord:
    :param autor:
    :param jednostka:
    :param zapisany_jako:
    :param typ_odpowiedzialnosci_skrot:
    :param kolejnosc:
    :return:
    """

    from bpp.models import Typ_Odpowiedzialnosci

    typ_odpowiedzialnosci = Typ_Odpowiedzialnosci.objects.get(
        skrot=typ_odpowiedzialnosci_skrot
    )

    if zapisany_jako is None:
        zapisany_jako = "%s %s" % (autor.nazwisko, autor.imiona)

    if kolejnosc is None:
        kolejnosc = klass.objects.filter(rekord=rekord).aggregate(Max("kolejnosc"))[
            "kolejnosc__max"
        ]
        if kolejnosc is None:
            kolejnosc = 0
        else:
            kolejnosc += 1

    inst = klass(
        rekord=rekord,
        autor=autor,
        jednostka=jednostka,
        typ_odpowiedzialnosci=typ_odpowiedzialnosci,
        kolejnosc=kolejnosc,
        zapisany_jako=zapisany_jako,
        dyscyplina_naukowa=dyscyplina_naukowa,
        afiliuje=afiliuje,
    )
    inst.full_clean()
    inst.save()
    return inst


class ModelZOpisemBibliograficznym(models.Model):
    """Mixin, umożliwiający renderowanie opisu bibliograficznego dla danego
    obiektu przy pomocy template."""

    tekst_przed_pierwszym_autorem = models.TextField(blank=True, null=True)
    tekst_po_ostatnim_autorze = models.TextField(blank=True, null=True)

    def opis_bibliograficzny(self, links=None):
        """Renderuje opis bibliograficzny dla danej klasy, używając:
        * w pierwszej kolejności zadeklarowanej Template dla danego typu rekordu (lub ogólnego Template),
        * w trzeciej kolejności templatki z dysku "opis_bibliograficzny/opis_bibliograficzny.html"

        :param links: "normal" lub "admin" jeżeli chcemy, aby autorzy prowadzili gdzieś (do stron browse/
        lub do admina).
        """

        template_name = SzablonDlaOpisuBibliograficznego.objects.get_for_model(self)
        if template_name is None:
            template_name = "opis_bibliograficzny.html"

        template = get_template(template_name)

        ret = (
            template.render(dict(praca=self, links=links))
            .replace("\r\n", "")
            .replace("\n", "")
        )
        while ret.find("  ") != -1:
            ret = ret.replace("  ", " ")

        return (
            ret.replace(" , ", ", ")
            .replace(" . ", ". ")
            .replace(". . ", ". ")
            .replace(". , ", ". ")
            .replace(" .", ".")
            .replace(".</b>[", ".</b> [")
        )

    def autorzy_dla_opisu(self):
        # Takie 'autorzy_set.all()' ale na potrzeby opisu bibliograficznego -- zaciąga
        # rekordy zależne za pomocą .select_related:

        return self.autorzy_set.select_related(
            "autor", "typ_odpowiedzialnosci"
        ).order_by("kolejnosc")

    def get_slug(self):
        if self.pk is None:
            return

        from bpp.util import slugify_function

        slug_tytul_oryginalny = slugify_function(self.tytul_oryginalny)

        slug_trzech_pierwszych_autorow = []
        for wyd_autor in self.autorzy_set.all().select_related("autor")[:3]:
            slug_trzech_pierwszych_autorow.append(
                f"{wyd_autor.autor.nazwisko} {wyd_autor.autor.imiona[:1]}"
            )
        slug_trzech_pierwszych_autorow = " ".join(slug_trzech_pierwszych_autorow)

        if hasattr(self, "zrodlo_id") and self.zrodlo_id is not None:
            slug_zrodla = slugify_function(self.zrodlo.nazwa)
        elif (
            hasattr(self, "wydawnictwo_nadrzedne")
            and self.wydawnictwo_nadrzedne_id is not None
        ):
            slug_zrodla = slugify_function(self.wydawnictwo_nadrzedne.tytul_oryginalny)
        else:
            slug_zrodla = ""

        lt, la, lz = (
            len(slug_tytul_oryginalny),
            len(slug_trzech_pierwszych_autorow),
            len(slug_zrodla),
        )

        if lt + la + lz >= 350:
            if lt > 200:
                lt = 200

            if lt + la + lz >= 350:
                if lz > 100:
                    lz = 100

                if lt + la + lz >= 350:
                    la = 50

        from django.contrib.contenttypes.models import ContentType

        ret = "-".join(
            [
                slug_tytul_oryginalny[:lt],
                slug_zrodla[:lz],
                slug_trzech_pierwszych_autorow[:la],
                str(ContentType.objects.get_for_model(self).pk),
                str(self.pk),
            ]
        )

        return slugify_function(ret)

    # Ten obiekt stanowi bazę do późniejszego zapełniania pól w podklasach. Pola,
    # które powinna podklasa definiować to:
    #
    # opis_bibliograficzny_cache = models.TextField(default="")
    # - generowane przez self.opis_bibliograficzny()
    #
    # opis_bibliograficzny_autorzy_cache = ArrayField(TextField(), blank=True, null=True)
    # -  To pole używane jest na ten moment jedynie przez moduł OAI, do szybkiego
    #    produkowania pola "Creator" dla formatu Dublin Core, vide moduł bpp.oai .
    #    To pole zawiera listę autorów, w kolejności, nazwisko i imię, bez
    #    tytułu
    #
    # slug = models.SlugField(max_length=400, unique=True, db_index=True, null=True, blank=True)
    # - skrót dla rekordu, dla SEO, zależy od m.in. tytułu, autorów, wydawnictwa nadrzędnego, źródła
    #
    # opis_bibliograficzny_zapisani_autorzy_cache = models.TextField(default="")
    # - zależy od klas autorów; to pole używane jest przez Raport autorów oraz Raport
    #   jednostek do szybkiego wypluwania listy zapisanych nazwisk
    #
    # def zaktualizuj_opis_bibliograficzny_cache(self, tylko_opis=False):
    #     autorzy = self.autorzy_dla_opisu()
    #     self.opis_bibliograficzny_cache = self.opis_bibliograficzny()
    #     self.slug = self._get_slug()
    #
    #     if hasattr(self, "autor"):
    #         zapisani = ["%s %s" % (autorzy[0].autor.nazwisko, autorzy[0].autor.imiona)]
    #     else:
    #         zapisani = [x.zapisany_jako for x in autorzy]
    #
    #     oac = ["%s %s" % (x.autor.nazwisko, x.autor.imiona) for x in autorzy]
    #     self.opis_bibliograficzny_autorzy_cache = oac
    #
    #     ozac = ", ".join(zapisani)
    #     self.opis_bibliograficzny_zapisani_autorzy_cache = ozac

    class Meta:
        abstract = True


class ZapobiegajNiewlasciwymCharakterom(models.Model):
    class Meta:
        abstract = True

    def clean_fields(self, *args, **kw):
        try:
            cf = self.charakter_formalny
        except ObjectDoesNotExist:
            cf = None

        if cf is not None:
            if self.charakter_formalny.skrot in ["D", "H", "PAT"]:
                raise ValidationError(
                    {
                        "charakter_formalny": [
                            safestring.mark_safe(
                                'Jeżeli chcesz dodać rekord o typie "%s"'
                                ', <a href="%s">kliknij tutaj</a>.'
                                % (
                                    self.charakter_formalny.nazwa,
                                    reverse(
                                        "admin:bpp_%s_add"
                                        % self.charakter_formalny.nazwa.lower().replace(
                                            " ", "_"
                                        )
                                    ),
                                )
                            )
                        ]
                    }
                )



# Register your models here.


from django.db.models import JSONField

# class JsonAdmin(admin.ModelAdmin):
from .mixins import ReadOnlyListChangeFormAdminMixin
from .widgets import PrettyJSONWidget

from django.contrib import admin


class BasePBNAPIAdminNoReadonly(admin.ModelAdmin):
    list_per_page = 25
    formfield_overrides = {JSONField: {"widget": PrettyJSONWidget}}


class BasePBNAPIAdmin(ReadOnlyListChangeFormAdminMixin, BasePBNAPIAdminNoReadonly):
    pass


class BaseMongoDBAdmin(BasePBNAPIAdmin):
    search_fields = ["mongoId", "versions"]
    list_filter = ["status", "verificationLevel"]
    readonly_fields = [
        "mongoId",
        "status",
        "verificationLevel",
        "verified",
        "created_on",
        "last_updated_on",
        # "versions",
    ]

    fields = readonly_fields + [
        "versions",
    ]

    # def get_search_results(self, request, queryset, search_term):
    #     queryset, use_distinct = super().get_search_results(
    #         request, queryset.exclude(status="DELETED"), search_term
    #     )
    #     return queryset, use_distinct



from django.db import models
from django.db.models import JSONField

from .base import BasePBNModel


class Language(BasePBNModel):
    code = models.CharField(max_length=5, primary_key=True)
    language = JSONField()

    def __str__(self):
        return self.language.get("pl") or self.language.get("en") or self.code



# Generated by Django 3.0.11 on 2021-01-01 16:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0229_nowe_sumy_status_korekty"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="metoda_do_roku_formularze",
            field=models.CharField(
                choices=[
                    ("jan_prev_then_current", "do stycznia poprzedni, potem obecny"),
                    ("max_rec", "najwiekszy rok rekordu w bazie"),
                ],
                default="jan_prev_then_current",
                help_text="Decyduje o sposobie wyświetlania maksymalnej daty 'Do roku' w formularzach. ",
                max_length=30,
                verbose_name="Data w polu 'do roku' w formularzach",
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-09-02 23:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0175_merge_20190824_0948'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charakter_formalny',
            name='charakter_sloty',
            field=models.PositiveSmallIntegerField(blank=True, choices=[(1, 'Książka'), (2, 'Rozdział')], default=None, help_text='Jak potraktować ten charakter przy kalkulacji slotów dla wydawnictwa zwartego?', null=True, verbose_name='Charakter dla slotów'),
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-08 20:22

from django.db import migrations

from bpp.models.autor import autor_split_string


def napraw_autorow_bez_nazwisk(apps, schema_editor):
    Autor = apps.get_model("bpp", "Autor")
    for elem in Autor.objects.filter(nazwisko=""):
        try:
            nazwisko, imiona = autor_split_string(elem.imiona)
        except ValueError:
            nazwisko, imiona = (elem.imiona, "[nie określono]")

        elem.nazwisko = nazwisko
        elem.imiona = imiona
        elem.save()


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0201_rekord_mat_z_www"),
    ]

    operations = [migrations.RunPython(napraw_autorow_bez_nazwisk)]
